/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * I/Os und Threads Beispielprogramm
 * ----------------------------------------------------------------------------
 * Dieses Programm demonstriert die Verwendung von Threads und einigen
 * I/O Funktionen wie z.B. das Abfragen des Start Tasters oder die ADCs 
 * auslesen.
 * Zunchst sind keine Threads aktiviert, bei jedem Druck auf den 
 * Start Taster wird ein weiterer Thread hinzugeschaltet bis alle 4 Demo     
 * Threads laufen. Dann werden bei jedem weiteren Druck auf den Start Taster
 * alle Threads nacheinander wieder deaktiviert.     
 * *****************************************************************************
 * Der Roboter bewegt sich in diesem Beispielprogramm NICHT und kann
 * am Rechner angeschlossen bleiben!
 ******************************************************************************/

#include "../../RP6CCLib/RP6CCLib.cc"

/**
 * Einfacher Thread der nur LED1 blinken lsst.
 */ 
void blink(void)
{
	while(true) // Jeder Thread hat seine eigene Endlosschleife! 
	{
	    // In den Programmen werden einfache Sachen wie der folgende 
	    // Abschnitt:
        Port_WriteBit(PORT_LED1, 0);
        Thread_Delay(25);
        Port_WriteBit(PORT_LED1, 1);
        Thread_Delay(25);
        
        // nicht weiter Kommentiert, da die Bedeutung schon aus den 
        // Funktionsaufrufen, Variablennamen und sonstigen Dingen
        // sofort ersichtlich ist. Diese einfachen Dinge sind auch 
        // bereits in den C-Control Beispielprogrammen mehr als ausreichend
        // dokumentiert.
	}
}

/**
 * Einfacher Thread der nur LED2 blinken lsst.
 */ 
void blink2(void)
{
	while(true)
	{
        Port_WriteBit(PORT_LED2, 0);
        Thread_Delay(80);
        Port_WriteBit(PORT_LED2, 1);
        Thread_Delay(20);
	}
}

/**
 * Hier wird pro Sekunde einmal ein kurzer Piepston erzeugt.
 */ 
void beep_1second(void)
{
	while(true)
	{
        beep_t(200,2,100); 
        // In Threads muss beep_t verwendet werden! 
        // Das normale beep verwendet AbsDelay, was nicht zusammen mit
        // Threads verwendet werden darf. 
	}
}

/**
 * Alle ADC Kanle der Reihe nach auslesen...
 */ 
void read_adcs(void)
{
    byte channel;
    char tmp[16];
    word result;

    channel = 0;
	while(true)
	{
        ADC_Set(ADC_VREF_VCC,channel);  // Kanal whlen...
        Thread_Delay(25);               // ... warten ...
        result = ADC_Read();            // ... und auslesen.

        // Messwert ausgeben:
        newline();
        print("ADC( ");
        Str_WriteInt(channel,tmp,0);
		Serial_WriteText(0,tmp);
        print(")=");
	    Str_WriteInt(result,tmp,0);
		Serial_WriteText(0,tmp);
        newline();

        // Nchster Kanal:
        channel++;
        if(channel > 7) channel = 0;
	}
}

/**
 * Warten bis der Starttaster gedrckt und wieder losgelassen worden ist.
 */ 
void press_start(void)
{
    newline();
    println("Press Start Button!");
    newline();

    while(true) {
		 if(!Port_ReadBit(PORT_START)) {
            beep_t(100, 5, 5); // Bei Tastendruck kurz piepsen
		 	while(!Port_ReadBit(PORT_START)) // Warten solange gedrckt... 
				Port_WriteBit(PORT_LED2, 1);
            Port_WriteBit(PORT_LED2, 0);
            break; 
		 }
	}
    beep_t(100, 5, 1);
    newline();
    println("OK!");
    newline();
}

void main(void)
{
	RP6_CCPRO_Init();

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println(" I/O & Threads Testprogramm  ");
    newline();

    showScreenLCD("I/O & Threads", "DEMO Program");
    AbsDelay(2000);
    showScreenLCD("...", "");

    setLEDs(LED1);

    beep(100, 200, 300);
    beep(300, 200, 100);
    beep(100, 200, 1);

    Port_WriteBit(PORT_LED2, 0);
	Port_WriteBit(PORT_LED1, 0);

    while(true)
    {
        showScreenLCD("I/O Testprogram", "Press Start!!!");
        press_start();                           // Start Taster drcken bitte!
        Thread_Start(1,blink); // Blinker Thread starten

        showScreenLCD("Blink 1 active  ", "Press Start!!!");
        press_start();
        Thread_Start(2,blink2); // Anderen Blinker Thread starten

        showScreenLCD("Blink 2 active  ", "Press Start!!!");
        press_start();
        Thread_Start(3,read_adcs); // ADC Test Thread starten

        showScreenLCD("ADC read active ", "Press Start!!!");
        press_start();
        Thread_Start(4,beep_1second); // Jede Sekunde Piepen Thread

        showScreenLCD("1sec Beep active", "Press Start!!!");
        press_start();

        //------------------------------------
        // Jetzt bei jedem Druck auf Start die Thread nacheinander
        // wieder beenden:

        showScreenLCD("Kill Thread     ", "Press Start!!!");

        setCursorPosLCD(0,12);
        writeCharLCD('1');
        press_start();
        Thread_Kill(1);

        setCursorPosLCD(0,12);
        writeCharLCD('2');
        press_start();
        Thread_Kill(2);

        setCursorPosLCD(0,12);
        writeCharLCD('3');
        press_start();
        Thread_Kill(3);

        setCursorPosLCD(0,12);
        writeCharLCD('4');
        press_start();
        Thread_Kill(4);

        Thread_Delay(20);
    }
}


