/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * Temperatursensor Beispielprogramm
 * ----------------------------------------------------------------------------
 * Der 12 Bit Temperatursensor auf dem Erweiterungsmodul kann wie in diesem
 * Programm gezeigt verwendet werden.
 * In der RP6CCLib sind zur Ansteuerung des Sensors einige Hilfsfunktionen
 * vorhanden. Dadurch wird das Auslesen der Temperaturwerte relativ einfach.
 *
 * *****************************************************************************
 * Der Roboter bewegt sich in diesem Beispielprogramm NICHT und kann
 * am Rechner angeschlossen bleiben!
 ******************************************************************************/


// WICHTIG: Immer die RP6CCLib mit einbinden:
#include "../../RP6CCLib/RP6CCLib.cc"

//------------------------------------------------------------------------------
// Die Adresse des Temperatursensors - diese kann ber die zwei Jumper auf
// dem Modul gendert werden! Standardmig ist sie auf 0x90 eingestellt sein.
#define TCN75_ADR 0x90


void main(void)
{
	RP6_CCPRO_Init(); // Auf Startsignal warten!

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("   Temperature Measurement Program ");
    newline();

    showScreenLCD("Temp. Sensor", "DEMO Program");
    AbsDelay(2000);

    beep(100, 200, 300);

    showScreenLCD("Temperature:", "        ");

    float temperature;  // in dieser Variablen wird
                        // der aktuelle Messwert gespeichert
    while(true)
    {
        // ---------------------------------
        // Temperatur lesen:
		TCN75_run(TCN75_ADR, TCN75A_CONFIG_RES_12);  // Messung starten...
        AbsDelay(250);                               // Pause
        TCN75_shutdown(TCN75_ADR);                   // Messung stoppen
        TCN75_read(TCN75_ADR);                       // Auslesen...

        // ---------------------------------
        char tmp[16],result[32];
        int temp_low;

		// Messwert in "float" Wert konvertieren (fr 12 Bit Messung):
		temp_low = getTemperatureLow();
		if(temp_low & 128)
			temp_low = (temp_low & 63) - 127;
	    else
			temp_low = temp_low & 63;
		temperature = temp_low + (0.0625 * (getTemperatureHigh()>>4));

		// Messwert ausgeben - auf dem LCD und ber die serielle Schnittstelle:
        newline();
		print("Temperature: ");
	    Str_WriteFloat(temperature,4,result,0); // float in Zeichenkette
		Serial_WriteText(0,result);             // umwandeln und ausgeben
        setCursorPosLCD(1,0);
        writeStringLCD(result);
        writeCharLCD(223);        //  "C" ausgeben
        writeCharLCD('C');
		print("C");
        newline();

        // Hier abfragen ob der Wert eine bestimmte Schwelle berschritten hat,
        // (hier 27.0C - kann natrlich gendert werden!)
        // dann ggf. Alarm auslsen:
        if (temperature > 27.0) {
            setLEDs(LED3 | LED4 | LED5);

            setCursorPosLCD(1,12);
            writeCharLCD('*');
            writeCharLCD('*');
            writeCharLCD('*');
            beep(200, 50, 100);
            setLEDs(0);
            clearPosLCD(1,12,3);
            beep(200, 50, 100);
        }
        else {
            setLEDs(LED1);
            AbsDelay(250);
        }
    }
}


