/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * Temperatursensor Beispielprogramm mit Threads
 * ----------------------------------------------------------------------------
 * Dieses Beispiel ist dem vorangegangenen Beispiel 04 sehr hnlich.
 * Allerdings werden hier Threads verwendet. Es wird hier gezeigt, dass
 * man bei Threads darauf achten muss, den Zugriff auf bestimmte Hardwaremodule
 * zu sperren wenn aus mehreren Threads darauf zugegriffen wird.
 * Andernfalls knnte es zu unvorhersagbarem Verhalten kommen.
 * In diesem Fall knnen nicht mehrere Threads (der Lese-Thread und der
 * Haupt-Thread ) gleichzeitig auf die LEDs und LC-Display zugreifen.
 * Wenn die Temperatur ber 27C steigen wrde, wrde Alarm ausgelst und
 * die LEDs angeschaltet und ein Text auf dem LCD angezeigt.
 * Das darf nicht gleichzeitig mit dem Anzeigen der Temperatur geschehen!
 *
 * Allgemein muss man bei der Verwendung von Threads immer auf solche
 * Dinge achten. Die meisten Hardware Komponenten knnen immer nur von genau
 * EINEM Thread zur selben Zeit verwendet werden!
 *
 * *****************************************************************************
 * Der Roboter bewegt sich in diesem Beispielprogramm NICHT und kann
 * am Rechner angeschlossen bleiben!
 ******************************************************************************/

// WICHTIG: Immer die RP6CCLib mit einbinden:
#include "../../RP6CCLib/RP6CCLib.cc"

// Die Adresse des Temperatursensors - diese kann ber die zwei Jumper auf
// dem Modul auch gendert werden (s. Anleitung)!
#define TCN75_ADR 0x90


// Der Temperatur Messwert wird in einer globalen Variablen gespeichert.
float temperature;


/**
 * Dieser der Thread ist fr das auslesen und ausgeben der Temperatur zustndig.
 */
void read_temperature(void)
{
	while(true)
	{
		// Temperatur lesen:
        Thread_Lock(1);  // Es knnen nicht mehrere Threads gleichzeitig
                        // auf den I2C Bus zugreifen.
            // Die bertragung wrde sonst gestrt.
            // Mit Thread_Lock(1) schaltet man also zunchst
            // das Multithreading aus ...
         // ... fhrt den kritischen Bereich aus...
		TCN75_run(TCN75_ADR, TCN75A_CONFIG_RES_12);
        Thread_Lock(0);    // ... und schaltet es wieder an!

        Thread_Delay(25); // Warten bis der Sensor fertig ist

        Thread_Lock(1);   // Noch ein kritischer Bereich:
        TCN75_shutdown(TCN75_ADR);
        TCN75_read(TCN75_ADR);
        Thread_Lock(0);

        // ---------------------------------
        char tmp[16],result[32];
        int temp_low;

		// Messwert in "float" Wert konvertieren (12 Bit Messung):
		temp_low = getTemperatureLow();
		if(temp_low & 128)
			temp_low = (temp_low & 63) - 127;
	    else
			temp_low = temp_low & 63;
		temperature = temp_low + (0.0625 * (getTemperatureHigh()>>4));

		// Messwert ausgeben - auf dem LCD und ber die serielle Schnittstelle:
        newline();
		print("Temperature: ");
	    Str_WriteFloat(temperature,4,result,0);
		Serial_WriteText(0,result);

        Thread_Lock(1);
            // Es knnen nicht mehrere Threads gleichzeitig
            // die setLEDs Funktion UND die LCD Funktionen
            // aufrufen, da beide auf den SPI Bus zugreifen.
            // Die bertragung wrde hier ebenfalls gestrt.

        setCursorPosLCD(1,0);
        writeStringLCD(result);
        writeCharLCD(223);
        writeCharLCD('C');
        Thread_Lock(0);

		print("C");
        newline();
		Thread_Delay(25);
	}
}


void main(void)
{
	RP6_CCPRO_Init(); // Auf Startsignal warten!

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("   Temperature Measurement Program  ");
    newline();

    showScreenLCD("Temp. Sensor", "DEMO Program 2");
    AbsDelay(2000);

    beep(100, 200, 300);

    showScreenLCD("Temperature:", "        ");

    // Thread zur Temperaturmessung starten:
    Thread_Start(1,read_temperature);

    while(true)
    {
        // Hier abfragen ob der Wert eine bestimmte Schwelle berschritten hat,
        // dann ggf. Alarm auslsen:
        if (temperature > 27.0) {

            Thread_Lock(1);         // Threads kurzzeitig sperren
            setLEDs(LED3 | LED4 | LED5);
            setCursorPosLCD(1,12);
            writeCharLCD('*');
            writeCharLCD('*');
            writeCharLCD('*');
            Thread_Lock(0);         // Threads wieder freigeben

            beep_t(200, 5, 10);     // Eine weitere Besonderheit bei der Verwendung
                                    // von Threads.
                                    // Wenn man Threads verwendet muss
                                    // man beep_t verwenden, sonst wrden alle
                                    // anderen Threads blockiert!
                                    // Bei beep_t muss man beachten das die Pausen
                                    // 10x lnger sind, da eine 10ms Zeitbasis
                                    // verwendet wird.
            Thread_Lock(1);
            setLEDs(0);
            clearPosLCD(1,12,3);
            Thread_Lock(0);

            beep_t(200, 5, 10);  // beep_t!
        }
        else {
            Thread_Lock(1);
            setLEDs(LED1);
            Thread_Lock(0);
        }
        Thread_Delay(25);
    }
}


