/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * I2C Bus Beispielprogramm
 * ----------------------------------------------------------------------------
 * Bevor das in den nchsten Beispielprogrammen etwas ausfhrlicher behandelt
 * wird, zeigt dieses Beispielprogramm noch kurz wie man die Status
 * Register ohne Threads auswerten kann. Die anderen Programme werden doch
 * schon recht umfangreich, daher zuvor noch dieses kleine einfachere Programm.
 *
 * Das Slave Programm, das auf dem Mainboard Controller luft, stellt
 * fr alle Sensoren Register zur Verfgung, die man per I2C auslesen kann.
 * Fr digitale Sensoren wie die Bumper und das ACS werden aber nur
 * einzelne Bits in einem Status Register gesetzt um anzuzeigen, dass etwas
 * detektiert wurde. Zustzlich wird bei jeder Zustandsnderung ein
 * Interrupt Signal gesetzt. So lsst sich ohne extra Abfrage ber den Bus
 * erkennen ob sich etwas gendert hat (geht schneller!).
 *
 * In diesem Beispielprogramm wird der Zustand der ACS und Bumper Sensoren
 * auf dem LCD und mit den LEDs dargestellt - der ACS Status wird mit den
 * LEDs auf dem Mainboard dargestellt und die Bumper mit den LEDs auf
 * dem CCPRO Erweiterungsmodul.
 *
 * Also stellen Sie den Roboter einfach auf den Tisch, bewegen die Hand
 * vor den IR Sensoren hin und her und drcken auf die Bumper!
 *
 * *****************************************************************************
 * Der Roboter bewegt sich in diesem Beispielprogramm NICHT und kann
 * am Rechner angeschlossen bleiben!
 ******************************************************************************/

// WICHTIG: Immer die RP6CCLib mit einbinden:
#include "../../RP6CCLib/RP6CCLib.cc"

// I2C Bus Adresse des Controllers auf dem Mainboard:
#define RP6_BASE_ADR 10
// (die Adresse kann auch im RP6I2CSlave Programm gendert werden)

void main(void)
{
	RP6_CCPRO_Init(); // Auf Startsignal warten!

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  I2C Testprogram ACS ");
    newline();

    beep(100, 200, 300);

    showScreenLCD("I2C-ACS-Bumper", "DEMO Program");
    AbsDelay(2000);
    clearLCD();

    // ACS auf mittlere Sendeleistung stellen:
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_MED);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, LED1 | LED4);  // Grne LEDs auf dem Mainboard
 
    while(true)
    {
        // Interrupt Leitung auswerten (diese wird bei Zustandnderungen
        // von wichtigen Sensoren wie dem ACS und den Bumpern gesetzt):
        if(Port_ReadBit(PORT_PE5_INT)) { // Hat der Controller auf dem Mainboard neue Daten?
            Port_WriteBit(PORT_LED2, 1); // LED2 signalisiert Zustandsnderung

            // -------------------------------------------------
            // Das Haupt-Status Register auslesen:
            byte status;
            status = RP6_readRegister(RP6_BASE_ADR, I2C_REG_STATUS1);

            // -------------------------------------------------
            // Die Bumper Status Variablen setzen:
            byte bumperLeft, bumperRight;
            bumperLeft = status & 0x2;
            bumperRight = status & 0x4;

            byte leds;
            leds = 0;

            if(bumperRight) {    // Rechter Tastsensor
                setCursorPosLCD(0,13);
                printLCD("BR");
                leds = LED5;
            }
            else  // Rechts nichts detektiert - also Text vom LCD lschen:
                clearPosLCD(0,13,2);

            if(bumperLeft) {  // Linker Tastsensor
                setCursorPosLCD(0,0);
                printLCD("BL");
                leds = leds | LED3;    // LED zustzlich aktivieren
            }
            else
                clearPosLCD(0,0,2);

            if(bumperLeft && bumperRight) {  // Beide, also in der Mitte
                setCursorPosLCD(0,6);
                printLCD("MID");
                leds = leds | LED4;
            }
            else
                clearPosLCD(0,6,3);

            // LEDs auf dem Erweiterungsmodul setzen:
            setLEDs(leds);
            // Diese Aufruf schaltet auch direkt die gelbe LED2 wieder
            // mit aus, die Zustandnderungen signalisiert.

            // -------------------------------------------------
            // Status des ACS mit den LEDs auf dem Mainboard
            // und mit dem LCD anzeigen:

            // Die ACS Status Variablen setzen:
            byte obstacleLeft, obstacleRight;
            obstacleLeft = status & 0x20;
            obstacleRight = status & 0x40;

            byte leds_rp6;
            leds_rp6 = 0;

            if(obstacleRight) {  // Hindernis rechts!
                leds_rp6 = LED3;
                setCursorPosLCD(1,11);
                printLCD("RIGHT");
            }
            else {               // Kein Hindernis.
                leds_rp6 = LED1;
                clearPosLCD(1,11,5) ;
            }

            if(obstacleLeft) {  // Hindernis links!
                leds_rp6 = leds_rp6 | LED6;  // zustzliche LED setzen!
                setCursorPosLCD(1,0);
                printLCD("LEFT");
            }
            else {               // Kein Hindernis.
                leds_rp6 = leds_rp6 | LED4;
                clearPosLCD(1,0,5);
            }

            if(obstacleRight && obstacleLeft) {   // Mitte!
                leds_rp6 = leds_rp6 | LED2 | LED5;
                setCursorPosLCD(1,6);
                printLCD("MID");
            }
            else
                clearPosLCD(1,6,3);

            // LEDs auf dem Mainboard setzen:
            RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, leds_rp6);
        }
    }
}


