//------------------------------------------------------------------------------
// Unkommentierte Version von RP6_Sensors.cc

#include "../../RP6CCLib/RP6CCLib.cc"

#define RP6_BASE_ADR 10

byte transmit_buffer[10]; 

void moveAtSpeed(byte desired_speed_left, byte desired_speed_right)
{
	RP6_writeCMD_2params(RP6_BASE_ADR, CMD_MOVE_AT_SPEED, 
                          desired_speed_left, desired_speed_right);
}


#define FWD 0
#define BWD 1
#define LEFT 2
#define RIGHT 3

void changeDirection(byte dir)
{
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_CHANGE_DIR, dir);
}

void stop(void)
{
	RP6_writeCMD(RP6_BASE_ADR, CMD_STOP);
}

word adcBat;  
word adcMotorCurrentLeft; 
word adcMotorCurrentRight; 
word adcLSL;             
word adcLSR;             
word adc0;              
word adc1;         

word mleft_speed; 
word mright_speed; 
word mleft_dist; 
word mright_dist; 
word mleft_des_speed; 
word mright_des_speed;  
word mleft_power;  
word mright_power; 

byte sensorBuf[24];

void getAllSensors(void)
{
	RP6_readRegisters(RP6_BASE_ADR, I2C_REG_POWER_LEFT, sensorBuf, 24);

	mleft_power = sensorBuf[0];
	mright_power = sensorBuf[1];
	mleft_speed = sensorBuf[2];
	mright_speed = sensorBuf[3];
	mleft_des_speed = sensorBuf[4];
	mright_des_speed = sensorBuf[5];
	mleft_dist = sensorBuf[6] + (sensorBuf[7]<<8);
	mright_dist = sensorBuf[8] + (sensorBuf[9]<<8);
	adcLSL = sensorBuf[10] + (sensorBuf[11]<<8);
	adcLSR = sensorBuf[12] + (sensorBuf[13]<<8);
	adcMotorCurrentLeft = sensorBuf[14] + (sensorBuf[15]<<8);
	adcMotorCurrentRight = sensorBuf[16] + (sensorBuf[17]<<8);
	adcBat = sensorBuf[18] + (sensorBuf[19]<<8);
	adc0 = sensorBuf[20] + (sensorBuf[21]<<8);
	adc1 = sensorBuf[22] + (sensorBuf[23]<<8);
}

void getLightSensors(void)
{
	RP6_readRegisters(RP6_BASE_ADR, I2C_REG_ADC_LSL_L, sensorBuf, 4);
	adcLSL = sensorBuf[0] + (sensorBuf[1]<<8);
	adcLSR = sensorBuf[2] + (sensorBuf[3]<<8);
}

#define RANGE 25

void readLightSensors(void)
{
    getLightSensors();

    newline();
    newline();
    print(" | LSL:"); printInteger(adcLSL);
    print(" | LSR:"); printInteger(adcLSR);
    newline();

    if(adcLSL > 999)
        adcLSL = 999;
    if(adcLSR > 999)
        adcLSR = 999;

    setCursorPosLCD(1, 2);
	printIntegerLCD(adcLSL);
    setCursorPosLCD(1, 7);

    int dif, abs_dif;
    dif = (adcLSL - adcLSR);
    if(dif > 160) { dif = 160; }
    if(dif < -160) { dif = -160; }
    if(dif < 0) { abs_dif = -dif; }
    else
        abs_dif = dif;
    if(abs_dif > 30) abs_dif = 30;

    if(adcLSL < (adcLSR - RANGE)) {
        writeCharLCD('<');
        changeDirection(RIGHT);  
        moveAtSpeed(abs_dif, abs_dif);
    }
    else if(adcLSR < (adcLSL - RANGE)) {
        writeCharLCD('>');
        changeDirection(LEFT);      
        moveAtSpeed(abs_dif, abs_dif);
    }
    else {
        writeCharLCD('='); stop();
    }

    setCursorPosLCD(1, 9);
    printIntegerLCD(adcLSR);

    if(dif > 140)
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x38);
    else if(dif > 75)
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x18);
    else if(dif > 35)
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x08);
    else if(dif > -35)
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x09);
    else if(dif > -75)
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x01);
    else if(dif > -140)
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x03);
    else
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x07);
}

void main(void)
{
	RP6_CCPRO_Init(); 

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 Sensor Reading 1 ");
    newline();

    beep(100, 200, 300);

    showScreenLCD("LIGHT SENSORS", "L             R");

    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x09);

    while(true)
    {
        readLightSensors();
        AbsDelay(10);
    }
}


