/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * I2C Bus - Sensorik - Beispielprogramm
 * ----------------------------------------------------------------------------
 * Dieses Beispielprogramm ist identisch zum Vorangegangenen,
 * allerdings wurden viele Funktionen in eine separate Datei (RP6MasterLib.cc)
 * ausgelagert um es bersichtlicher zu gestalten und den Code leichter
 * wiederverwenden zu knnen. Die RP6MasterLib.cc ist in diesem
 * Beispiel noch nicht vollstndig - sie wird in den nchsten Beispielen
 * noch deutlich erweitert!
 *
 * Es ist natrlich allgemein zu empfehlen, grere Projekte immer
 * in mehrere Dateien aufzuteilen - wenn das Programm mal mehr als tausend
 * Zeilen hat, verliert man sonst leicht den berblick. 
 * 
 * ############################################################################ 
 * #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+
 * 
 * ACHTUNG: Der Roboter bewegt sich in diesem Beispielprogramm! 
 * Bitte stellen Sie sicher, dass gengend Freiraum um den Roboter 
 * vorhanden ist (in diesem Programm dreht er sich nur auf der Stelle, also 
 * reichen 20cm Abstand auf allen Seiten)!   
 *
 * Achten Sie darauf das Flachbandkabel des USB Interfaces vom Roboter zu 
 * trennen wenn sich der Roboter zu weit vom PC entfernen kann. 
 * 
 * Sie brauchen NICHT die Verbindung des Interfaces zum PC zu trennen! Nur 
 * das Flachbandkabel zum Roboter muss entfernt werden!
 *
 * #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+
 * ############################################################################
 ******************************************************************************/

#include "../../RP6CCLib/RP6CCLib.cc"

// ###########################
// WICHTIG: 
// Die neue Library einbinden:
#include "RP6MasterLib.cc"
// In diese Datei wurden viele Funktionen ausgelagert!


 // Maximaler Differenzwert der Lichtsensoren bei
// dem noch nichts unternommen wird:
#define RANGE 25

/**
 * Hier wird demonstriert wie die Lichtsensoren ausgelesen werden
 * knnen.  Die Funktion ist identisch zum vorangegangenen Beispiel.
 *
 */
void readLightSensors(void)
{
    // Lichtsensoren auslesen:
    getLightSensors();

    // Werte ausgeben:
    newline();
    newline();
    print(" | LSL:"); printInteger(adcLSL);
    print(" | LSR:"); printInteger(adcLSR);
    newline();

    // Werte limitieren um 3 stellig zu bleiben:
    if(adcLSL > 999)
        adcLSL = 999;
    if(adcLSR > 999)
        adcLSR = 999;

    // Linken Sensorwert auf dem LCD ausgeben:
    setCursorPosLCD(1, 2);
	printIntegerLCD(adcLSL);
    setCursorPosLCD(1, 7);

    // Absolute Differenz ermitteln und begrenzen:
    int dif, abs_dif;
    dif = (adcLSL - adcLSR);
    if(dif > 160) { dif = 160; }
    if(dif < -160) { dif = -160; }
    if(dif < 0) { abs_dif = -dif; }
    else
        abs_dif = dif;
    if(abs_dif > 30) abs_dif = 30;

    // Welcher Sensor ist heller? Ist der Unterschied gro genug?
    if(adcLSL < (adcLSR - RANGE)) { // Rechts ist es heller - also nach rechts drehen:
        writeCharLCD('<');
        changeDirection(RIGHT);
        moveAtSpeed(abs_dif, abs_dif);
    }
    else if(adcLSR < (adcLSL - RANGE)) { // Links ist es heller.
        // Also nach links drehen:
        writeCharLCD('>');
        changeDirection(LEFT);
        moveAtSpeed(abs_dif, abs_dif);
    }
    else { // Helligkeitsunterschied zwischen den Sensoren ist klein, also stoppen:
        writeCharLCD('='); stop();
    }

    // Rechten Sensorwert auf dem LCD ausgeben:
    writeCharLCD(' ');
    writeCharLCD(' ');
    printIntegerLCD(adcLSR);

    // Bargraph Display mit den LEDs auf dem Mainboard:
    if(dif > 140)
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x38);
    else if(dif > 75)
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x18);
    else if(dif > 35)
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x08);
    else if(dif > -35)
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x09);
    else if(dif > -75)
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x01);
    else if(dif > -140)
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x03);
    else
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x07);
}

void main(void)
{
	RP6_CCPRO_Init(); // Auf Startsignal warten!

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 Sensor Reading 2 ");
    newline();

    beep(100, 200, 300);

    // Dieser Text bleibt immer sichtbar,
    // nur die Sensorwerte werden aktualisiert.
    showScreenLCD("LIGHT SENSORS", "L             R");

    // LEDs auf dem Mainboard setzen (die beiden grnen LEDs):
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x09);

    while(true)
    {
        // In der Hauptschleife wird stndig die Funktion
        // zur berprfung der Lichtsensoren aufgerufen:
        readLightSensors();
        AbsDelay(50);
    }
}


