/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * RP6 Master Library
 * ----------------------------------------------------------------------------
 * In diese Datei werden immer wieder bentigte Funktionen ausgelagert um
 * das Hauptprogramm bersichtlicher zu halten.  
 ******************************************************************************/

//------------------------------------------------------------------------------
//

#ifndef __MASTER_LIB__     // Alles nur EINMAL einbinden!
#define __MASTER_LIB__


// I2C Bus Adresse des Controllers auf dem Mainboard:
#define RP6_BASE_ADR 10


/*****************************************************************************/
// Sensoren/ADCs:

// Die einzelnen ADC Kanle bekommen je eine eigene Variable:
word adcBat;                // Akkuspannung
word adcMotorCurrentLeft;   // Motorstrom Links (Messwert!)
word adcMotorCurrentRight;  // Motorstrom Rechts (Messwert!)
word adcLSL;                // Lichtsensor Links
word adcLSR;                // Lichtsensor Rechts
word adc0;                  // Analog/Digital Wandler Kanal 0
word adc1;                  // Analog/Digital Wandler Kanal 1

// Gemessene Geschwindigkeiten (Messwert! In Zhlschritten pro 1/5 Sekunde):
word mleft_speed;  // Links
word mright_speed; // Rechts

// Zurckgelegte Distanz (Messwert!)
word mleft_dist;  // Links
word mright_dist; // Rechts

// Gewnschte Geschwindigkeit (kein Messwert!):
word mleft_des_speed;   // Links
word mright_des_speed;  // Rechts

// Power
word mleft_power;   // Links
word mright_power;  // Rechts

// ACS 
byte obstacleLeft; // Hindernis links
byte obstacleRight; // Hindernis rechts

// Tastesensoren
byte bumperLeft;   // Kollision links
byte bumperRight;  // Kollision rechts


byte messageBuf[24];

/**
 * Die obigen Variablen aktualisieren sich leider nicht von allein.
 * Man muss dazu diese Funktion aufrufen. Sie liest alle aktuellen Sensorwerte
 * ber den I2C Bus aus.
 *
 * Natrlich dauert das eine gewisse Zeit. Man sollte das also nur so oft
 * wie unbedingt notwendig tun!
 * Bei wichtigen nderungen (Kollision, Hindernis erkannt, Akku leer) wird ausserdem
 * eine Interrupt Leitung gesetzt.
 */
void getAllSensors(void)
{
    RP6_readRegisters(RP6_BASE_ADR, I2C_REG_POWER_LEFT, messageBuf, 24);
    mleft_power = messageBuf[0];
    mright_power = messageBuf[1];
    mleft_speed = messageBuf[2];
    mright_speed = messageBuf[3];
    mleft_des_speed = messageBuf[4];
    mright_des_speed = messageBuf[5];
    mleft_dist = messageBuf[6] + (messageBuf[7]<<8);
    mright_dist = messageBuf[8] + (messageBuf[9]<<8);
    adcLSL = messageBuf[10] + (messageBuf[11]<<8);
    adcLSR = messageBuf[12] + (messageBuf[13]<<8);
    adcMotorCurrentLeft = messageBuf[14] + (messageBuf[15]<<8);
    adcMotorCurrentRight = messageBuf[16] + (messageBuf[17]<<8);
    adcBat = messageBuf[18] + (messageBuf[19]<<8);
    adc0 = messageBuf[20] + (messageBuf[21]<<8);
    adc1 = messageBuf[22] + (messageBuf[23]<<8);
}

/**
 * Im Gegensatz zur getAllSensors Funktion, liest getLightSensors nur die Werte der zwei
 * Lichtsensoren vom Roboter aus.
 */
void getLightSensors(void)
{
	RP6_readRegisters(RP6_BASE_ADR, I2C_REG_ADC_LSL_L, messageBuf, 4);
	adcLSL = messageBuf[0] + (messageBuf[1]<<8);
	adcLSR = messageBuf[2] + (messageBuf[3]<<8);
}



/*****************************************************************************/
// Rotations Funktion

byte transmit_buffer[10]; // temporrer Buffer fr die I2C Kommunikation


 /**
 * Diese Funktion setzt die Motorgeschwindigkeit auf einen bestimmten Wert.
 * Dieser wird dann fortlaufend eingeregelt (d.h. der Roboter versucht
 * die Geschwindigkeit mglichst konstant zu halten.)
 * Die Geschwindigkeit wird in "Encoder Zhlschritten" pro 0.2 Sekunden
 * eingestellt (s. a. RP6 Anleitung!).
 *
 * Die zurckgelegte Strecke pro Zhlschritt variiert von Roboter zu Roboter
 * aufgrund der hohen Auflsung der Encoder sprbar. Man muss durch
 * abfahren von Teststrecken bestimmter Lnge die tatschliche Auflsung
 * ermitteln.
 * Meist liegt diese im Bereich von 0.235mm bis 0.25mm pro Zhlschritt.
 * Die Encoder liefern 625 Zhlschritte pro Radumdrehung.
 *
 * Der Controller auf dem Mainboard aktualisiert die gemessenen
 * Geschwindigkeitswerte standardmig alle 200ms = 0.2s also
 * mit einer Frequenz von 5Hz.
 *
 * Die tatschliche Geschwindigkeit errechnet sich bei einer
 * Auflsung von 0.25mm daher wie folgt:
 * speed = encoder_value * 0.25mm * 5 Hz  =
 *       = encoder_value * 1.25mm *   1/s = encoder_value * 0.125cm/s.
 * "encoder_value" ist hier die Zahl der Impulse die von den Encodern
 * zurckgeliefert werden.
 * Um aus einer gemessenen Geschwindigkeit in Encoder Zhlschritten
 * also eine Geschwindigkeit in cm/s abzuleiten, muss man einfach den Messwert
 * mit 0.125 multiplizieren (sofern die Encoder Auflsung 0.25mm ist)
 *
 *
 * Beispiel fr moveAtSpeed: Sie mchten den Roboter mit 10cm/s
 * fahren lassen.
 * Also:
 * 10cm/s = encoder_value * 0.125cm/s
 * => encoder_value = 10 / 0.125 = 80
 *
 * Im Programm schreibt man dann also:
 * moveAtSpeed(80,80); // Mit etwa 10cm/s geradeaus fahren!
 *
 */
void moveAtSpeed(byte desired_speed_left, byte desired_speed_right)
{
	RP6_writeCMD_2params(RP6_BASE_ADR, CMD_MOVE_AT_SPEED, 
                         desired_speed_left, desired_speed_right);
}



// Status des aktuellen Bewegungsablaufs (Rotation oder Strecke abfahren)
byte movementComplete;


// Bewegungsrichtungen:
#define FWD 0
#define BWD 1
#define LEFT 2
#define RIGHT 3

#define BLOCKING 1

/**
 * Diese Funktion veranlasst den Roboter mit einer bestimmten Geschwindigkeit
 * (desired_speed) in eine vorgegebene Richtung (dir) um einen bestimmten
 * Winkel (angle) auf der Stelle zu rotieren.
 * Mit dem Parameter "blocking" kann bestimmt werden, ob die Funktion
 * blockieren soll bis die Bewegung vom Slave Controller als abgeschlossen
 * gemeldet wurde. Dies ist fr einfache Bewegungssequenzen hilfreich! 
 *
 * Beispiel:
 * RP6_rotate(80,LEFT,100, BLOCKING); // mit 10cm/s um 100 nach links rotieren
 * RP6_rotate(40,RIGHT,50, BLOCKING); // mit 5cm/s um 50 nach rechts rotieren
 *
 */
void RP6_rotate(byte desired_speed, byte dir, word angle, byte blocking)
{
    transmit_buffer[0] = desired_speed;
    transmit_buffer[1] = dir;
    transmit_buffer[2] = ((angle>>8) & 0xFF);
    transmit_buffer[3] = (angle & 0xFF);
    RP6_writeCommand_params(RP6_BASE_ADR, CMD_ROTATE,
                             transmit_buffer, 4);
    // Warten bis die Bewegung abgeschlossen ist (sofern gewartet werden
    // soll - das kann mit dem Parameter Blocking gesteuert werden!): 
    while(blocking && !movementComplete) {

    }
}

/**
 * Diese Funktion veranlasst den Roboter mit einer bestimmten Geschwindigkeit
 * (desired_speed) in eine vorgegebene Richtung (dir) um einen bestimmten
 * Winkel (angle) auf der Stelle zu rotieren.
 * Mit dem Parameter "blocking" kann bestimmt werden, ob die Funktion
 * blockieren soll bis die Bewegung vom Slave Controller als abgeschlossen
 * gemeldet wurde. Dies ist fr einfache Bewegungssequenzen hilfreich!
 *
 * Beispiel:
 * RP6_rotate(80,LEFT,100, BLOCKING); // mit 10cm/s um 100 nach links rotieren
 * RP6_rotate(40,RIGHT,50, BLOCKING); // mit 5cm/s um 50 nach rechts rotieren
 *
 */
void RP6_move(byte desired_speed, byte dir, word distance, byte blocking)
{
    transmit_buffer[0] = desired_speed;
    transmit_buffer[1] = dir;
    transmit_buffer[2] = ((distance>>8) & 0xFF);
    transmit_buffer[3] = (distance & 0xFF);
    RP6_writeCommand_params(RP6_BASE_ADR, CMD_MOVE,
                             transmit_buffer, 4);
    // Warten bis die Bewegung abgeschlossen ist:
    while(blocking && !movementComplete) {

    }
}

/**
 * Die (dauerhafte) Bewegungsrichtung des Roboters wird mit 
 * dieser Funktion gendert.
 *  
 * Beispiel:
 * changeDirection(LEFT);   // nach links drehen
 * changeDirection(BWD);    // Rckwrts
 * changeDirection(RIGHT);  // Rechts
 * changeDirection(FWD);    // Vorwrts
 */
void changeDirection(byte dir)
{
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_CHANGE_DIR, dir);
}

/**
 * Die aktuelle Bewegung des Roboter kann man mit dieser Funktion sofort stoppen.
 */
void stop(void)
{
	RP6_writeCMD(RP6_BASE_ADR, CMD_STOP);
}



#endif
