/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * I2C Bus - Sensorik - Beispielprogramm
 * ----------------------------------------------------------------------------
 * Nun erweitern wir das vorangegangene Beispiel um einige weitere
 * Funktionalitten. Die empfangenen RC5 Daten von TV Fernbedienungen werden
 * nach Adresse, Daten und Togglebit ausgewertet. 
 * (dazu muss die Fernbedienung natrlich RC5 Code senden! s. a. RP6 Anleitung!) 
 *  
 * Der Watchdog den das I2CSlave Programm im Controller auf dem Mainboard 
 * bereitstellt wird genutzt und die "Requests" (Anfragen) auf dem 
 * LCD dargestellt.
 * Mit dem Watchdog kann man sofort sehen, ob sich das Programm auf einem der
 * beiden Controller aufgehangen hat.  
 *  
 * *****************************************************************************
 * Der Roboter bewegt sich in diesem Beispielprogramm NICHT und kann
 * am Rechner angeschlossen bleiben!
 ******************************************************************************/

//------------------------------------------------------------------------------
//

#include "../../RP6CCLib/RP6CCLib.cc"

// ###########################
// Die neue Library einbinden - ACHTUNG - diese hat sich in diesem
// Beispielprogramm gendert, da einige neue Funktionalitten hinzugekommen
// sind!
#include "RP6MasterLib.cc"


#define MASK_WDT 0x4            // Watchdog      (1 Bit)
#define MASK_WDT_RQ 0x8         // Watchdog Request     (1 Bit)
byte heartbeat;                 // "Herzschlag" Variable - zur Anzeige eines
                                // blinkenden Symbols auf dem LCD (s.u.).


// Variablen fr die Daten des RC5 Empfngers:
byte RC5_toggle;
byte RC5_adr;
byte RC5_data;

void thread_checkINT0(void)
{
    while(true) {
        if(Port_ReadBit(PORT_PE5_INT)) { // Hat der Controller auf dem Mainboard neue Daten?
            Port_WriteBit(PORT_LED2, 1);

            // Die drei Status Register auslesen:
            RP6_readRegisters(RP6_BASE_ADR, 0, messageBuf, 3);

            // Wir berprfen, ob die Status Bits sich verndert haben,
            // also verknpfen wir den alten und neuen Wert per Exklusiv-Oder
            // um nur die genderten Bits auswerten zu mssen.
            byte compare;
            compare = messageBuf[0] ^ interrupt_status;

            interrupt_status = messageBuf[0];

            // ------------------------------------
            // Zunchst prfen wir ob sich die ACS Status Bits verndert haben.
            // Falls nicht, hat irgend etwas anderes den Interrupt ausgelst und
            // wir knnen mit den anderen Abfragen weitermachen.
            if(compare & MASK_ACS)
            {
                 beep_t(200, 2, 2); // Ganz kurz Piepsen
                 // Die ACS Status Variablen setzen:
                 obstacleLeft = messageBuf[0] & 0x20;
                 obstacleRight = messageBuf[0] & 0x40;

                 // -------
                 // Der folgende Codeabschnitt ist nur fr das setzten der
                 // LEDs und der Darstellung auf dem LCD da.
                 // Wenn man nur obstacleLeft und Right bentigt, kann dieser
                 // Teil also gelscht werden.
                 byte leds_rp6;
                 leds_rp6 = 0;

                 if(obstacleRight) {  // Hindernis rechts!
                     leds_rp6 = LED3;
                     setCursorPosLCD(1,11);
                     printLCD("RIGHT");
                 }
                 else {               // Kein Hindernis.
                     leds_rp6 = LED1;
                     clearPosLCD(1,11,5) ;
                 }

                 if(obstacleLeft) {  // Hindernis links!
                     leds_rp6 = leds_rp6 | LED6;  // zustzliche LED setzen!
                     setCursorPosLCD(1,0);
                     printLCD("LEFT");
                 }
                 else {               // Kein Hindernis.
                     leds_rp6 = leds_rp6 | LED4;
                     clearPosLCD(1,0,5);
                 }

                 if(obstacleRight && obstacleLeft) {   // Mitte!
                     leds_rp6 = leds_rp6 | LED2 | LED5;
                     setCursorPosLCD(1,6);
                     printLCD("MID");
                 }
                 else
                     clearPosLCD(1,6,3);

                 // LEDs auf dem Mainboard setzen:
                 RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, leds_rp6);
                 // -------
            }


            // ------------------------------------
            // Abfragen ob sich der Zustand der Bumper gendert hat, falls
            // ja, Text auf Display anzeigen und LEDs setzen:
            if(compare & MASK_BUMPER)
            {
                 beep_t(100, 1, 1);
                 bumperLeft = messageBuf[0] & 0x2;
                 bumperRight = messageBuf[0] & 0x4;

                 byte leds;
                 leds = 0;

                 if(bumperRight) {    // Rechter Tastsensor
                     setCursorPosLCD(0,13);
                     printLCD("BR");
                     leds = LED5;
                 }
                 else  // Rechts nichts detektiert - also Text vom LCD lschen:
                     clearPosLCD(0,13,2);

                 if(bumperLeft) {  // Linker Tastsensor
                     setCursorPosLCD(0,0);
                     printLCD("BL");
                     leds = leds | LED3;    // LED zustzlich aktivieren
                 }
                 else
                     clearPosLCD(0,0,2);

                 if(bumperLeft && bumperRight) {  // Beide, also in der Mitte
                     setCursorPosLCD(0,6);
                     printLCD("MID");
                     leds = leds | LED4;
                 }
                 else
                     clearPosLCD(0,6,3);

                 // LEDs auf dem Erweiterungsmodul setzen:
                 setLEDs(leds);
            }

            // Alles weitere wird nur auf der seriellen Schnittstelle ausgegeben,
            // denn die LEDs sind ja schon alle mit ACS und Bumpern belegt ;)


            // ------------------------------------
            // berprfen ob ein RC5 Datenpaket mit dem IRCOMM empfangen
            // worden ist - falls ja auf der seriellen ausgeben:
            if(interrupt_status & MASK_RC5)
            {
                RP6_readRegisters(RP6_BASE_ADR, I2C_REG_RC5_ADR, messageBuf, 2);
                RC5_toggle = messageBuf[0] >> 5;
                RC5_adr = messageBuf[0] & 0x1F;
                RC5_data = messageBuf[1];

                // Ton erzeugen. Tonhhe abhngig vom Tastencode:
                beep_t((RC5_data+5)*15, 2, 2); 
                
                // Empfangene Daten ausgeben:
                print(" RC5: ");
                printInteger(RC5_toggle);   // Togglebit (Taste gehalten
                print(":");                 // oder mehrfach gedrckt?)
                printInteger(RC5_adr);      // Adresse
                print(":");
                printInteger(RC5_data);     // Tastencode
                println(" ");
            }

            // ------------------------------------
            // berprfen ob die Batteriespannung zu niedrig ist:
            if(compare & MASK_BATLOW)
            {
                RP6_readRegisters(RP6_BASE_ADR, I2C_REG_ADC_UBAT_L, messageBuf, 2);
                adcBat = (messageBuf[1] << 8) | (messageBuf[0]);
                print("BAT LOW:");
                printInteger(adcBat);
                println(" V");
            }

            // ------------------------------------
            // Status des Antriebssystems berprfen:
            if(compare & MASK_DRIVE)
            {
                 print(" DRIVE:");
                 printInteger(messageBuf[2]);
                 println(" ");
                 // Bewegung abgeschlossen?
                 movementComplete = messageBuf[2] & 128;
            }

            // ------------------------------------
            // Auf Watchdog Requests prfen:
            if(messageBuf[1] & MASK_WDT_RQ)
            {
                // Jeden Request durch ein/ausblenden
                // eines Symbols auf dem Display anzeigen ("Herzschlag"):
                if(heartbeat)
                {
                    clearPosLCD(0, 11, 1);
                    heartbeat = false;
                }
                else
                {
                    setCursorPosLCD(0,11);
                    printLCD("*");
                    heartbeat = true;
                }
            }
        }
        else
            Port_WriteBit(PORT_LED2, 0);
        Thread_Delay(1);
    }
}


void main(void)
{
	RP6_CCPRO_Init(); // Auf Startsignal warten!

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 Sensor Reading 4 ");
    newline();

    beep(100, 200, 300);

    showScreenLCD("Sensors", "DEMO Program 4");
    AbsDelay(2000);
    clearLCD();

    // ACS auf mittlere Sendeleistung stellen:
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_MED);

    // Watchdog und Watchdog Request aktivieren:
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT, true);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT_RQ, true);


    // Der Thread zur berprfung von Statusnderungen wird gestartet:
    Thread_Start(1,thread_checkINT0);

    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, LED1 | LED4);
    
    // Nichts tun:
    while(true)
    {
        Thread_Delay(100);
    }
}


