//------------------------------------------------------------------------------
// Unkommentierte Version von RP6_Sensors4.cc

#include "../../RP6CCLib/RP6CCLib.cc"

#include "RP6MasterLib.cc"


#define MASK_WDT 0x4 
#define MASK_WDT_RQ 0x8 
byte heartbeat; 


byte RC5_toggle;
byte RC5_adr;
byte RC5_data;

void thread_checkINT0(void)
{
    while(true) {
        if(Port_ReadBit(PORT_PE5_INT)) {
            Port_WriteBit(PORT_LED2, 1);

            RP6_readRegisters(RP6_BASE_ADR, 0, messageBuf, 3);

            byte compare;
            compare = messageBuf[0] ^ interrupt_status;
            interrupt_status = messageBuf[0];

            if(compare & MASK_ACS)
            {
                 beep_t(200, 2, 2); 
                
                 obstacleLeft = messageBuf[0] & 0x20;
                 obstacleRight = messageBuf[0] & 0x40;

                 byte leds_rp6;
                 leds_rp6 = 0;

                 if(obstacleRight) { 
                     leds_rp6 = LED3;
                     setCursorPosLCD(1,11);
                     printLCD("RIGHT");
                 }
                 else {   
                     leds_rp6 = LED1;
                     clearPosLCD(1,11,5) ;
                 }

                 if(obstacleLeft) {
                     leds_rp6 = leds_rp6 | LED6; 
                     setCursorPosLCD(1,0);
                     printLCD("LEFT");
                 }
                 else {       
                     leds_rp6 = leds_rp6 | LED4;
                     clearPosLCD(1,0,5);
                 }

                 if(obstacleRight && obstacleLeft) {
                     leds_rp6 = leds_rp6 | LED2 | LED5;
                     setCursorPosLCD(1,6);
                     printLCD("MID");
                 }
                 else
                     clearPosLCD(1,6,3);

                 RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, leds_rp6);
            }

            if(compare & MASK_BUMPER)
            {
                 beep_t(100, 1, 1);
                 bumperLeft = messageBuf[0] & 0x2;
                 bumperRight = messageBuf[0] & 0x4;

                 byte leds;
                 leds = 0;

                 if(bumperRight) {
                     setCursorPosLCD(0,13);
                     printLCD("BR");
                     leds = LED5;
                 }
                 else  
                     clearPosLCD(0,13,2);

                 if(bumperLeft) {
                     setCursorPosLCD(0,0);
                     printLCD("BL");
                     leds = leds | LED3; 
                 }
                 else
                     clearPosLCD(0,0,2);

                 if(bumperLeft && bumperRight) {
                     setCursorPosLCD(0,6);
                     printLCD("MID");
                     leds = leds | LED4;
                 }
                 else
                     clearPosLCD(0,6,3);

                 setLEDs(leds);
            }

            if(interrupt_status & MASK_RC5)
            {
                RP6_readRegisters(RP6_BASE_ADR, I2C_REG_RC5_ADR, messageBuf, 2);
                RC5_toggle = messageBuf[0] >> 5;
                RC5_adr = messageBuf[0] & 0x1F;
                RC5_data = messageBuf[1];

                beep_t((RC5_data+5)*15, 2, 2); 
                
                print(" RC5: ");
                printInteger(RC5_toggle);
                print(":"); 
                printInteger(RC5_adr); 
                print(":");
                printInteger(RC5_data);
                println(" ");
            }

            if(compare & MASK_BATLOW)
            {
                RP6_readRegisters(RP6_BASE_ADR, I2C_REG_ADC_UBAT_L, messageBuf, 2);
                adcBat = (messageBuf[1] << 8) | (messageBuf[0]);
                print("BAT LOW:");
                printInteger(adcBat);
                println(" V");
            }

            if(compare & MASK_DRIVE)
            {
                 print(" DRIVE:");
                 printInteger(messageBuf[2]);
                 println(" ");
                 
                 movementComplete = messageBuf[2] & 128;
            }

            if(messageBuf[1] & MASK_WDT_RQ)
            {
                if(heartbeat)
                {
                    clearPosLCD(0, 11, 1);
                    heartbeat = false;
                }
                else
                {
                    setCursorPosLCD(0,11);
                    printLCD("*");
                    heartbeat = true;
                }
            }
        }
        else
            Port_WriteBit(PORT_LED2, 0);
        Thread_Delay(1);
    }
}

void main(void)
{
	RP6_CCPRO_Init();

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 Sensor Reading 4 ");
    newline();

    beep(100, 200, 300);

    showScreenLCD("Sensors", "DEMO Program 4");
    AbsDelay(2000);
    clearLCD();

    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_MED);

    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT, true);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT_RQ, true);

    Thread_Start(1,thread_checkINT0);

    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, LED1 | LED4);
    
    while(true)
    {
        Thread_Delay(100);
    }
}




