/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * I2C Bus - Bewegung & Sensorik - Beispielprogramm
 * ----------------------------------------------------------------------------
 * Ein weiteres einfaches Bewegungsprogramm, diesmal reagiert der RP6
 * allerdings auf die Bumper.
 * Zunchst fhrt der RP6 vorwrts in einer Kreisbahn.
 * Kollidiert er mit einem Hindernis, werden die Motoren gestoppt und
 * die LEDs fangen an zu blinken.
 * (man knnte natrlich auch anders auf die Kollision reagieren...)
 ******************************************************************************/


#include "../../RP6CCLib/RP6CCLib.cc"

// ###########################
#include "RP6MasterLib.cc"

// ###########################
#include "RP6_Sensorcheck.cc"

/**
 * Dieser Thread wird ausgefhrt sobald der Roboter mit einem
 * Hindernis kollidiert ist.
 */
void blink(void)
{
    while(true)
    {
        setRP6LEDs(LED2 | LED3);
        setLEDs(LED3 | LED4 | LED5);  // LEDs kurz aufblitzen lassen ...
        Thread_Delay(5);              // ...
        setLEDs(0);                   // ...
        Thread_Delay(30);
        setRP6LEDs(LED5 | LED6);
        setLEDs(LED3 | LED4 | LED5);
        Thread_Delay(5);
        setLEDs(0);
        Thread_Delay(30);
    }
}


void main(void)
{
	RP6_CCPRO_Init(); // Auf Startsignal warten!

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 Movement DEMO 2 ");
    newline();

    beep(100, 200, 300);

    showScreenLCD("Movement", "DEMO Program 2");
    AbsDelay(2000);
    clearLCD();

    // ACS auf mittlere Sendeleistung stellen:
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_MED);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT, true);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT_RQ, true);

    // Der Thread zur berprfung von Statusnderungen wird gestartet:
    Thread_Start(1,thread_checkINT0);

    // Vorwrts im Kreis fahren:
    changeDirection(FWD);
    moveAtSpeed(100,40);

    setCursorPosLCD(1,0);
    printLCD("Moving...    :-)");

    setRP6LEDs(LED1 | LED4);

    byte stopped;
    stopped = false;

    while(true)
    {
        if(!stopped) { // Es sollte nur einmal stop() und Thread_Start aufgerufen
                       // werden, daher fragen wir hier ab, ob das bereits
                       // geschehen ist.

            // Man muss die Bumper nicht in RP6_Sensorcheck.cc
            // auswerten. Man kann auch hier direkt die beiden Variablen
            // der Tastsensoren abfragen:
            if(bumperLeft || bumperRight)
            {
                stop();                 // Roboter stoppen!
                Thread_Start(2,blink);  // Blinken!
                stopped = true;  // Roboter wurde angehalten!
                setCursorPosLCD(1,0);
                printLCD("Ouch! Crash! :-(");
            }
        }
        Thread_Delay(1); // Muss immer hier rein, sonst werden die anderen
                         // Threads blockiert.
    }
}


