/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * RP6 - Subsumption - Beispielprogramm
 * ----------------------------------------------------------------------------
 * Nun kommen wir zu einem etwas interessanteren Beispiel. Wir verwenden hier
 * die von Rodney Brooks in the 1980er Jahren verffentlichte "Subsumptions
 * Architektur" um einen verhaltensbasierten Roboter zu realisieren.
 *
 * s. a. (originale Verffentlichung in Englisch):
 * http://people.csail.mit.edu/brooks/papers/AIM-864.pdf
 *
 * s. a. (kurze Zusammenfassung in Deutsch):
 * http://www.igi.tugraz.at/STIB/WS98/Bergmann/einleitung.htm
 *
 * Wie auch schon in der RP6 Anleitung beschrieben, kann man mit diesem Prinzip
 * den Roboter dazu bringen, sich wie eine Art Insekt zu verhalten.
 * Je mehr und je bessere Sensoren man dem "Insekt" dabei zur Verfgung stellt,
 * desto komplexer knnen die Verhaltensweisen werden und desto "intelligenter"
 * verhlt es sich von aussen betrachtet. Natrlich geht das nur in gewissen
 * Grenzen (Rechenleistung und Speicherkapazitt, Energiebedarf und Gewicht...)
 *
 * In diesem Beispiel werden wir zunchst nur die "Fhler" unseres
 * primitiven Insekts verwenden - die beiden Bumper/Tastsensoren.
 *
 * Zwei Verhaltensweisen reichen dazu bereits aus: "Cruise" und "Escape".
 * "Cruise" tut eigentlich gar nichts. Es gibt nur konstant den
 * Befehl "Fahre Vorwrts!".
 * Und genau das tut der Roboter auch, sofern nicht irgendein anderes
 * Verhalten die Ausgabe von "Cruise" unterdrckt.
 *
 * Das kann in diesem Beispiel nur das Verhalten "Escape". Es hat die hhere
 * Prioritt und unterdrckt die Ausgaben von "Cruise", sobald einer der
 * Bumper einen Zusammensto meldet.
 * Je nachdem ob der Zusammensto links, rechts oder in der Mitte passiert ist,
 * werden unterschiedliche Ausweichmanver eingeleitet.
 * Grundstzlich fhrt der Roboter einfach immer ein Stck rckwrts, dreht
 * auf der Stelle nach links oder rechts und fhrt weiter.
 * Das ist natrlich sehr primitiv und kann noch weiter verbessert werden.
 * Was in den nachfolgenden Beispielprogrammen auch getan wird (dieses Programm
 * hat nmlich noch einige Schwchen wie man schnell feststellen wird wenn
 * man es ausprobiert)
 *
 * Man muss sich dabei allerdings auch eins klar machen:
 * Wenn man selber in einem Auto mit komplett undurchsichtigen Fensterscheiben
 * sitzten wrde und einzig und allein an der Front des Autos zwei Taster htte,
 * die im inneren zwei Lampen an und ausschalten wenn man damit gegen ein
 * Hindernis stt... wie wrde man damit in einer
 * unbekannten Umgebung navigieren?
 * Drfte relativ schwierig werden damit von Zuhause aus zum
 * Supermarkt zu finden ;-)
 *
 * Hinweis:
 * Schauen Sie sich bitte auch die Beschreibungen der Beispiele in der normalen
 * RP6 Anleitung an, auch wenn diese nicht fr das C-Control PRO Modul
 * geschrieben wurden. Die allgemeine Beschreibung in der Anleitung ist auch
 * zum Verstndnis dieses Programms hilfreich.
 * Anders als auf dem Controller des Mainboards steht uns auf dem CCPRO
 * Multithreading zur Verfgung. Wir mssen hier also nicht explizit die
 * einzelnen Verhalten in einer Schleife aufrufen - das wird automatisch vom
 * Multithreading Scheduler erledigt.
 *
 * Als Anfnger sollte man sich viel Zeit nehmen um dieses Programm
 * nachzuvollziehen, vieles ist eventuell nicht sofort auf den ersten Blick
 * verstndlich. Schauen Sie sich auch auf jeden Fall mal die Variante dieses
 * Programms OHNE Kommentare an!
 * Das gilt auch fr alle nachfolgenden Programme,
 * die noch etwas komplexer werden!
 ******************************************************************************/

#include "../../RP6CCLib/RP6CCLib.cc"

// ###########################
#include "RP6MasterLib.cc"

// ###########################
#include "RP6_Sensorcheck.cc"


// Bei vielen Threads macht es Sinn, die Thread Nummern durch lesbare
// Namen zu ersetzen:
#define THREAD_CHECK_INT0               1
#define THREAD_CHECK_BUMPERS            2
#define THREAD_BEHAVIOUR_CONTROLLER     3

// Namen fr Verhaltens Threads:
#define BEHAVIOUR_CRUISE   4
#define BEHAVIOUR_ESCAPE   5


// Damit die Programme mit mehreren Verhalten spter bersichtlich bleiben,
// definieren wir fr jedes Verhalten ein Befehls-Array mit dem KOMPLETT alle
// Steuerungswerte bzw. Befehle an den Roboter weitergegeben werden knnen.
// Man kann auch einzelne Variablen dazu verwenden und das kann
// teilweise sogar notwendig werden, sofern man mal Fliepunktzahlen oder
// hnliches bergeben muss. Im Rahmen dieser Beispielprogramme wird das
// jedoch nicht notwendig sein und ein einfaches byte Array reicht aus.
// Die folgenden Konstanten sind fr ALLE diese Befehls-Arrays identisch,
// damit der Programmcode gut lesbar bleibt und man mit NAMEN auf die
// Feldelemente zugreifen kann:
#define STATE           0     // Zustand des Verhaltens
#define SPEED_LEFT      1     // Geschwindigkeitswert linker Motor
#define SPEED_RIGHT     2     // Geschwindigkeitswert rechter Motor
#define MOVE_DIR        3     // Bewegungsrichtung
// Anm: (Leider bietet CompactC noch keine Structs, sonst htte man es
// besser damit realisieren knnen)

// Global definieren wieviele Felder unser Kommando-Array hat:
#define COMMAND_SIZE    4
// (da kommen in den nchsten Beispielen noch einige dazu, daher hier
// an zentraler Stelle... )

// Fr das STATE Feld des Kommando Arrays gibt es fr jedes Verhalten
// zwei mgliche Zustnde:
#define IDLE    0    // Ruhezustand
#define ACTIVE  1    // Aktiv
// ABER: In den Verhalten werden meist noch weitere Zustnde hinzukommen!
// Fr den Kommando Interpreter ist aber nur wichtig, ob einer Verhalten
// Aktiv ist oder ob es gerade nichts zu melden hat (IDLE).


// -----------------------------------------------------------------------------
// Das CRUISE Verhalten:


// Das Kommando-Array fr das "Cruise" Verhalten:
byte cruise[COMMAND_SIZE];

// Bewegungsgeschwindigkeit fr "Cruise" Verhalten:
#define CRUISE_SPEED 70

/**
 * Dieses Verhalten ist sehr einfach. Der Roboter bewegt sich
 * schlicht und ergreifend einfach nur vorwrts.
 */
void behaviour_cruise(void)
{
    // Kommando-Array initialisieren:
    cruise[STATE] = ACTIVE; // Verhalten ist immer aktiv.
    cruise[SPEED_LEFT] = CRUISE_SPEED;
    cruise[SPEED_RIGHT] = CRUISE_SPEED;
    cruise[MOVE_DIR] = FWD;

    // In diesem Verhalten passiert sonst nichts weiter:
    while(true)
    {
        Thread_Delay(100);
    }
}


// -----------------------------------------------------------------------------
// Das ESCAPE Verhalten:

byte escape[COMMAND_SIZE];

// Geschwindigkeiten als Konstanten definieren:
#define ESCAPE_MOVE_SPEED 60
#define ESCAPE_TURN_SPEED 50

#define ESCAPE_SPEED_BWD    40
#define ESCAPE_SPEED_ROTATE 30

// Einige zustzliche Zustnde fr unser Escape-Verhalten:
#define ESCAPE_FRONT		2
#define ESCAPE_LEFT  		3
#define ESCAPE_RIGHT	    4

/**
 * Dieses Verhalten realisiert eine einfache Strategie um Hindernisse mithilfe
 * der Tastsensoren zu umfahren.
 *
 * Natrlich kollidiert der Roboter hiermit zunchst mit den Hindernissen,
 * das werden wir in einem spteren Beispielprogramm mithilfe des ACS
 * vermeiden. Allerdings ist das nicht immer zuverlssig, daher ist dieses
 * Verhalten auch damit noch sehr wichtig!
 *
 * In diesem Beispielprogramm luft hier alles noch zeitgesteuert ab - das
 * wird spter noch verbessert.
 */
void behaviour_escape(void)
{
    static word time_counter, move_time;

    // Kommando-Array initialisieren (nichts tun!):
    escape[STATE] = IDLE; // Verhalten ist inaktiv!
    escape[SPEED_LEFT] = 0;
    escape[SPEED_RIGHT] = 0;
    escape[MOVE_DIR] = FWD;

    while(true)
    {
        // Tastsensoren berprfen und dementsprechend
        // den Zustand wechseln:
        if(bumperLeft && bumperRight)   // Mitte!
        {
            // Zustand auf "Frontalzusammensto" ndern:
            escape[STATE] = ESCAPE_FRONT;
            move_time = 400; // 4 Sekunden Bewegungen
        }
        else if(bumperLeft)             // Links
        {
            // Falls nicht schon erkannt wurde das BEIDE
            // Tastsensoren aktiviert waren, kann der
            // Zustand auf "Zusammensto Links" gendert werden:
            if(escape[STATE] != ESCAPE_FRONT)
            {
                escape[STATE] = ESCAPE_LEFT;
                move_time = 300;  // 3 Sekunden Bewegungen
            }
            // Wenn dieser Zustand aktiviert wird,
            // ist der Roboter mit dem linken Bumper
            // gegen ein Hindernis gestoen. Dass muss aber
            // noch nichts heissen, es kann sein, dass kurze
            // Zeit spter doch noch ESCAPE_FRONT aktiviert
            // wird. Der rechte Bumper knnte ein ganz klein
            // wenig spter aktiviert werden...
            // Mit der Abfrage oben if(escape[STATE] != ESCAPE_FRONT)
            // wird sichergestellt, dass auch der Zustand FRONT gehalten
            // wird wenn einer der Bumper wieder inaktiv wird.
        }
        else if(bumperRight)            // Rechts
        {
            // Ebenso fr die rechte Seite:
            if(escape[STATE] != ESCAPE_FRONT)
            {
                escape[STATE] = ESCAPE_RIGHT;
                move_time = 300;
            }
        }


        if(escape[STATE] != IDLE) // Verhalten ist aktiv? Dann Ausweichen...
        {
              // Der Thread wird nach jedem Durchlauf fr etwa 10ms pausiert.
              // (auch lnger wenn andere Threads laufen! Das ist also SEHR
              // ungenau und nur als grobe Timing Funktion verwendbar!)
              // Wir zhlen bei jedem Durchlauf einfach eine Timer-Variable
              // hoch um fr bestimmte Zeitintervalle verschiedene Bewegungen
              // durchzufhren...

              if(time_counter < move_time) // "move_time * 10ms" lang Rckwrts fahren!
              {
                  escape[SPEED_LEFT] = ESCAPE_SPEED_BWD;   // Geschwindigkeit setzen
                  escape[SPEED_RIGHT] = ESCAPE_SPEED_BWD;
                  escape[MOVE_DIR] = BWD;                  // Rckwrts fahren
              }
              else if(time_counter < (2*move_time)) // weitere "move_time * 10ms" lang rotieren!
              {
                  escape[SPEED_LEFT] = ESCAPE_SPEED_ROTATE;   // Geschwindigkeit setzen
                  escape[SPEED_RIGHT] = ESCAPE_SPEED_ROTATE;

                  // Je nachdem wo die Kollision stattgefunden hat, wird
                  // die Ausweichrichtung gesetzt:
                  if(escape[STATE] == ESCAPE_FRONT)  // Frontal...
                  {
                        escape[MOVE_DIR] = LEFT;
                        // Hier knnte man auch intelligenter ausweichen...
                        // z.B. abhngig davon welcher Bumper zuletzt aktiv
                        // war o....
                  }
                  else if(escape[STATE] == ESCAPE_LEFT) // Links...
                  {
                        escape[MOVE_DIR] = RIGHT;
                  }
                  else if(escape[STATE] == ESCAPE_RIGHT) // Rechts...
                  {
                        escape[MOVE_DIR] = LEFT;
                  }
              }
              else  // Nach insgesamt (2 * move_time * 10ms) ...
              {
                  escape[STATE] = IDLE; // wieder zurck in den Ruhezustand!
              }

              time_counter++;  // Zhlvariable fr die Zeit...
        }
        else
            time_counter = 0;

        Thread_Delay(1);  // 10ms warten
    }
}


// -----------------------------------------------------------------------------
// Die Kontroll Routinen fr die Koordination der Verhalten:

/**
 * Dieser Funktion kann eines der Kommando Arrays der verschiedenen
 * Verhalten bergeben werden. Anhand der gesetzten Variablenwerte
 * werden dann die passenden Befehle an den Controller auf dem
 * Mainboard weitergeleitet. In den nchsten Beispielen wird das
 * noch erweitert! In diesem ersten Beispiel wird einfach nur
 * der aktuelle Geschwindigkeitswert bertragen und die Richtung gesetzt.
 */
void moveCommand(byte command[])
{
    byte speed_l, speed_r, dir;

    speed_l = command[SPEED_LEFT];
    speed_r = command[SPEED_RIGHT];
    dir = command[MOVE_DIR];

    changeDirection(dir);
    moveAtSpeed(speed_l,speed_r);
}

/**
 * Dieser Thread realisiert die "Unterdrckung" von Verhaltensweisen
 * niedriger Prioritt durch solche mit hherer Prioritt. Im Prinzip sind
 * es nur ein paar if-else Abfragen hintereinander.
 * Ganz wichtig ist hierbei, dass KEIN Verhalten direkt auf Aktoren des
 * Roboters zugreift! Aktoren sind dabei z.B. Motoren, LEDs, LC-Display usw.
 * Alle Verhalten legen nur die ihrer Meinung nach passenden Befehle
 * in globalen Kommando-Arrays ab. Dieser Thread whlt die Befehle genau
 * EINES der Verhalten aus und leitet sie an den Roboter weiter.
 * Dabei wird stets das Verhalten mit der hchsten Prioritt gewhlt.
 *
 */
void behaviourController(void)
{
    while(true)
    {
        Thread_Lock(1); // Threadumschaltung sperren!
        // Das ist wichtig damit nicht whrend die Kommandos ausgefhrt
        // werden noch etwas an den Kommando Arrays gendert wird.

        // Hier werden die einzelnen Verhalten der Reihe nach abgefragt.
        // Nochmal die Erluterung: Es kann nur EINES der Verhalten die
        // Kontrolle ber den Roboter bernehmen!
        // Wenn mehrere Verhalten aktiv sind, entscheidet die Prioritt
        // der Verhalten welches die Kontrolle bekommt.
        // Da hier ein if-else-if-else Konstrukt verwendet wird, hat das
        // oberste Verhalten auch die hchste Prioritt.
        // In diesem Fall hat "escape" also die hchste Prioritt, gefolgt
        // von "cruise". Ist gar kein Verhalten aktiv, werden einfach nur
        // die Motoren gestoppt. Das kann in diesem Beispiel aber gar nicht
        // eintreffen (knnte man allerdings tun, z.B. wenn die Akkus fast
        // leer sind)
        if(escape[STATE] != IDLE)      // Prioritt 2  - "escape"
        {
            moveCommand(escape);
        }
        else if(cruise[STATE] != IDLE) // Prioritt 1  - "cruise"
        {
            moveCommand(cruise);
        }
        else                           // Prioritt 0  - Nichts tun
        {
            stop();
        }

        Thread_Lock(0);  // Threadumschaltung wieder freigeben.
        Thread_Delay(5); // 50ms Pause
        // Die 50ms Pause verhindert zu schnelle Wechsel zwischen verschiedenen
        // Verhaltensweisen. Kann aber bei Bedarf angepasst werden!
    }
}


// -------------------------------------------------------------------------


void main(void)
{
	RP6_CCPRO_Init(); // Auf Startsignal warten!

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 Subsumption DEMO 1 ");
    newline();

    beep(100, 200, 300);

    showScreenLCD("Subsumption", "DEMO Program 1");
    AbsDelay(2000);
    clearLCD();

    // ACS auf mittlere Sendeleistung stellen:
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_MED);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT, true);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT_RQ, true);

    // Der Thread zur berprfung von Statusnderungen wird gestartet:
    Thread_Start(THREAD_CHECK_INT0,thread_checkINT0);

    // Die Threads der Verhaltensweisen starten:
    Thread_Start(BEHAVIOUR_CRUISE,behaviour_cruise);
    Thread_Start(BEHAVIOUR_ESCAPE,behaviour_escape);

    // Kontroll Thread fr die Verhalten (Arbiter) starten:
    Thread_Start(THREAD_BEHAVIOUR_CONTROLLER,behaviourController);

    setCursorPosLCD(1,0);
    printLCD("Moving...    :-)");

    while(true)
    {
        Thread_Delay(10);
    }
}


