//------------------------------------------------------------------------------
// Unkommentierte Version von RP6_Behaviour.cc

#include "RP6MasterLib.cc"

#include "RP6_Sensorcheck.cc"

#define THREAD_CHECK_INT0               1
#define THREAD_CHECK_BUMPERS            2
#define THREAD_BEHAVIOUR_CONTROLLER     3

#define BEHAVIOUR_CRUISE   4
#define BEHAVIOUR_ESCAPE   5

#define STATE           0
#define SPEED_LEFT      1
#define SPEED_RIGHT     2
#define MOVE_DIR        3

#define COMMAND_SIZE    4

#define IDLE    0
#define ACTIVE  1


byte cruise[COMMAND_SIZE];
#define CRUISE_SPEED 60

void behaviour_cruise(void)
{
    cruise[STATE] = ACTIVE; 
    cruise[SPEED_LEFT] = CRUISE_SPEED;
    cruise[SPEED_RIGHT] = CRUISE_SPEED;
    cruise[MOVE_DIR] = FWD;

    while(true)
    {
        Thread_Delay(100);
    }
}


byte escape[COMMAND_SIZE];

#define ESCAPE_MOVE_SPEED 50
#define ESCAPE_TURN_SPEED 40

#define ESCAPE_SPEED_BWD    40
#define ESCAPE_SPEED_ROTATE 30

#define ESCAPE_FRONT		1
#define ESCAPE_LEFT  		2
#define ESCAPE_RIGHT	    3

void behaviour_escape(void)
{
    static word time_counter, move_time;

    escape[STATE] = IDLE;
    escape[SPEED_LEFT] = 0;
    escape[SPEED_RIGHT] = 0;
    escape[MOVE_DIR] = FWD;

    while(true)
    {
        if(bumperLeft && bumperRight)
        {
            escape[STATE] = ESCAPE_FRONT;
            move_time = 400;
        }
        else if(bumperLeft)
        {
            if(escape[STATE] != ESCAPE_FRONT)
            {
                escape[STATE] = ESCAPE_LEFT;
                move_time = 300;
            }
        }
        else if(bumperRight)
        {
            if(escape[STATE] != ESCAPE_FRONT)
            {
                escape[STATE] = ESCAPE_RIGHT;
                move_time = 300;
            }
        }
        
        if(escape[STATE] != IDLE)
        {
              if(time_counter < move_time)
              {
                  escape[SPEED_LEFT] = ESCAPE_SPEED_BWD;
                  escape[SPEED_RIGHT] = ESCAPE_SPEED_BWD;
                  escape[MOVE_DIR] = BWD; 
              }
              else if(time_counter < (2*move_time))
              {
                  escape[SPEED_LEFT] = ESCAPE_SPEED_ROTATE; 
                  escape[SPEED_RIGHT] = ESCAPE_SPEED_ROTATE;

                  if(escape[STATE] == ESCAPE_FRONT)
                  {
                        escape[MOVE_DIR] = LEFT;
                  }
                  else if(escape[STATE] == ESCAPE_LEFT)
                  {
                        escape[MOVE_DIR] = RIGHT;
                  }
                  else if(escape[STATE] == ESCAPE_RIGHT)
                  {
                        escape[MOVE_DIR] = LEFT;
                  }
              }
              else
              {
                  escape[STATE] = IDLE;
              }
              time_counter++;
        }
        else
            time_counter = 0;

        Thread_Delay(1); 
    }
}


void moveCommand(byte command[])
{
    byte speed_l, speed_r, dir;

    speed_l = command[SPEED_LEFT];
    speed_r = command[SPEED_RIGHT];
    dir = command[MOVE_DIR];

    changeDirection(dir);
    moveAtSpeed(speed_l,speed_r);
}

void behaviourController(void)
{
    while(true)
    {
        Thread_Lock(1); 
        
        if(escape[STATE] != IDLE)
        {
            moveCommand(escape);
        }
        else if(cruise[STATE] != IDLE)
        {
            moveCommand(cruise);
        }
        else 
        {
            stop();
        }

        Thread_Lock(0);
        Thread_Delay(5); 
    }
}

void main(void)
{
	RP6_CCPRO_Init();

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 Subsumption DEMO 1 ");
    newline();

    beep(100, 200, 300);

    showScreenLCD("Subsumption", "DEMO Program 1");
    AbsDelay(2000);
    clearLCD();

    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_MED);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT, true);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT_RQ, true);

    Thread_Start(THREAD_CHECK_INT0,thread_checkINT0);

    Thread_Start(BEHAVIOUR_CRUISE,behaviour_cruise);
    Thread_Start(BEHAVIOUR_ESCAPE,behaviour_escape);

    Thread_Start(THREAD_BEHAVIOUR_CONTROLLER,behaviourController);

    setCursorPosLCD(1,0);
    printLCD("Moving...    :-)");

    while(true)
    {
        Thread_Delay(10);
    }
}
