/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * RP6 - Subsumption - Beispielprogramm 2
 * ----------------------------------------------------------------------------
 * Jetzt werden wir das relativ einfache Escape Verhalten weiter verbessern.
 * Rein zeitbasierte Bewegungen sind ja nicht immer erwnscht...
 * Der Roboter kann mit seinen Drehgebern auch ganz bestimmte Strecken
 * abfahren und um bestimmte Winkel rotieren!
 * Dazu mssen wir unser Kommando Array etwas erweitern und die Auswertung
 * dessen natrlich auch.
 * Bei jeder Kollision wird in diesem Programm nun ein bestimmter
 * Bewegungsablauf durchgefhrt - also z.B. bei einem Frontalzusammensto, wird
 * 300mm weit zurckgesetzt und um 110 auf der Stelle rotiert.
 * Dazu ist anstatt des Zeitablaufs ein kleiner Zustandsautomat in das
 * Escape Verhalten eingebaut worden...
 ******************************************************************************/

#include "../../RP6CCLib/RP6CCLib.cc"

// ###########################
#include "RP6MasterLib.cc"

// ###########################
#include "RP6_Sensorcheck.cc"


// Bei vielen Threads macht es Sinn, die Thread Nummern durch lesbare
// Namen zu ersetzen:
#define THREAD_CHECK_INT0               1
#define THREAD_CHECK_BUMPERS            2
#define THREAD_BEHAVIOUR_CONTROLLER     3

// Namen fr Verhaltens Threads:
#define BEHAVIOUR_CRUISE   4
#define BEHAVIOUR_ESCAPE   5


// Neben den bereits bekannten Feldern fr das Kommando-Array:
#define STATE           0     // Zustand des Verhaltens
#define SPEED_LEFT      1     // Geschwindigkeitswert linker Motor
#define SPEED_RIGHT     2     // Geschwindigkeitswert rechter Motor
#define MOVE_DIR        3     // Bewegungsrichtung
// kommen nun noch ein paar weitere hinzu:
#define ACTION          4     // Bewegungsaktion (Rotieren, bestimmte Distanz)
#define MOVE_VALUE_L    5     // Distanz oder Winkel, je nachdem ob es eine
#define MOVE_VALUE_H    6     // normale Bewegung oder eine Rotation ist
// Es sind dafr ZWEI Bytes erforderlich, um Werte grer als 255 darstellen
// zu knnen. In diesem Fall muss der Wert in zwei Bytes aufgeteilt werden.
// (High Byte mit "(X >> 8) & 0xFF" und Low Byte mit (X & 0xFF))

// Global definieren wieviele Felder unser Kommando-Array hat:
#define COMMAND_SIZE    8
// (da kommen in den nchsten Beispielen noch einige dazu, daher hier
// an zentraler Stelle... )

// Fr das STATE Feld des Kommando Arrays gibt es fr jedes Verhalten
// zwei mgliche Zustnde:
#define IDLE    0
#define ACTIVE  1
// ABER: In den Verhalten werden meist noch weitere Zustnde hinzukommen!
// Fr den Kommando Interpreter ist aber nur wichtig, ob ein Verhalten
// Aktiv ist oder ob es gerade nichts zu melden hat (IDLE).



// Das neue ACTION Feld, kann folgende Zustnde annehmen:
#define NO_ACTION          0      // Keine Aktion
#define ACTION_ROTATE      1      // Rotieren
#define ACTION_MOVE        2      // Bestimmte Distanz abfahren



// -----------------------------------------------------------------------------
// Das CRUISE Verhalten:

// Das Kommando-Array fr das "Cruise" Verhalten:
byte cruise[COMMAND_SIZE];

// Bewegungsgeschwindigkeit fr das "Cruise" Verhalten:
#define CRUISE_SPEED 80

/**
 * Dieses Verhalten ist sehr einfach. Der Roboter bewegt sich
 * schlicht und ergreifend einfach nur vorwrts.
 */
void behaviour_cruise(void)
{
    // Kommando-Array initialisieren:
    cruise[STATE] = ACTIVE; // Verhalten ist immer aktiv.
    cruise[SPEED_LEFT] = CRUISE_SPEED;
    cruise[SPEED_RIGHT] = CRUISE_SPEED;
    cruise[MOVE_DIR] = FWD;
    // Neu hinzugekommene Felder initialisieren (nichts tun...):
    cruise[ACTION] = NO_ACTION;
    cruise[MOVE_VALUE_L] =  0;
    cruise[MOVE_VALUE_H] =  0;

    // In diesem Verhalten passiert sonst nichts weiter:
    while(true)
    {
        Thread_Delay(100);
    }
}


// -----------------------------------------------------------------------------
// Das ESCAPE Verhalten:

byte escape[COMMAND_SIZE];

// Geschwindigkeiten als Konstanten definieren:
#define ESCAPE_SPEED_BWD    70
#define ESCAPE_SPEED_ROTATE 60

#define ESCAPE_FRONT		1
#define ESCAPE_LEFT  		2
#define ESCAPE_RIGHT	    3

/**
 * Dieses Verhalten realisiert eine einfache Strategie um Hindernisse mithilfe
 * der Tastsensoren zu umfahren.
 *
 * Natrlich kollidiert der Roboter hiermit zunchst mit den Hindernissen.
 * Dass werden wir in einem spteren Beispielprogramm mithilfe des ACS
 * vermeiden. Allerdings ist das nicht immer zuverlssig, daher ist dieses
 * Verhalten auch damit noch sehr wichtig!
 *
 * In diesem Beispiel hat das Verhalten aber noch einen groen Nachteil,
 * es luft stets gleich ab - das wird im nchsten Beispielprogramm noch
 * behoben!
 *
 */
void behaviour_escape(void)
{
    // Kommando-Array initialisieren (nichts tun!):
    escape[STATE] = IDLE; // Verhalten ist inaktiv!
    escape[SPEED_LEFT] = 0;
    escape[SPEED_RIGHT] = 0;
    escape[MOVE_DIR] = FWD;
    escape[ACTION] = NO_ACTION;
    escape[MOVE_VALUE_L] =  0;
    escape[MOVE_VALUE_H] =  0;

    static word next_move_value;
    static byte next_direction;

    // Zustnde des Bewegungsablaufs:
    #define MOVE_BACKWARDS 0
    #define ROTATE         1
    #define WAIT_END       2
    static byte move_state;

    while(true)
    {
        // Tastsensoren berprfen und dementsprechend den Zustand wechseln:
        if(bumperLeft && bumperRight)   // Mitte!
        {
            // Zustand auf "Frontalzusammensto" ndern:
            escape[STATE] = ESCAPE_FRONT;
            move_state = MOVE_BACKWARDS; // Zustand der Bewegung zurcksetzen!
            next_move_value = 300; // 300mm Rckwrts fahren...
        }
        else if(bumperLeft)             // Links
        {
            // Falls nicht schon erkannt wurde das BEIDE
            // Tastsensoren aktiviert waren, kann der
            // Zustand auf "Zusammensto Links" gendert werden:
            if(escape[STATE] != ESCAPE_FRONT)
            {
                escape[STATE] = ESCAPE_LEFT;
                move_state = MOVE_BACKWARDS;
                next_move_value = 180; // 180mm
            }
            // Wenn dieser Zustand aktiviert wird,
            // ist der Roboter mit dem linken Bumper
            // gegen ein Hindernis gestoen. Das muss aber
            // noch nichts heissen, es kann sein, dass kurze
            // Zeit spter doch noch ESCAPE_FRONT aktiviert
            // wird. Der rechte Bumper knnte ein ganz klein
            // wenig spter aktiviert werden...
            // Mit der Abfrage oben if(escape[STATE] != ESCAPE_FRONT)
            // wird sichergestellt, dass auch der Zustand FRONT gehalten
            // wird wenn einer der Bumper wieder inaktiv wird.
        }
        else if(bumperRight)            // Rechts
        {
            // Ebenso fr die rechte Seite:
            if(escape[STATE] != ESCAPE_FRONT)
            {
                escape[STATE] = ESCAPE_RIGHT;
                move_state = MOVE_BACKWARDS;
                next_move_value = 180; // 180mm
            }
        }

        if(escape[STATE] != IDLE) // Verhalten ist aktiv? Dann Ausweichen...
        {
              // Zustandsautomat des Bewegungsablaufs:
              switch(move_state)
              {
                    case MOVE_BACKWARDS: // Rckwrts bewegen...
                        escape[SPEED_LEFT] = ESCAPE_SPEED_BWD;   // Geschwindigkeit setzen
                              // Das LINKE Geschwindigkeitsfeld gilt fr BEIDE Motoren wenn
                              // eine Bewegungsaktion durchgefhrt werden soll!
                        escape[MOVE_DIR] = BWD;                  // Rckwrts fahren

                        // Zu fahrende Strecke in Millimetern einstellen:
                        escape[MOVE_VALUE_L] =  next_move_value & 0xFF;
                        escape[MOVE_VALUE_H] =  (next_move_value >> 8) & 0xFF;
                        // Es gibt hier zwei Bytes, da grere Werte als 255
                        // mglich sein sollen!
                        escape[ACTION] = ACTION_MOVE;  // Aktion: Bestimmte Distanz bewegen
                        // SOBALD diese Variable gesetzt wurde, wird die Aktion
                        // eingeleitet! ALSO MERKE: escape[ACTION] IMMER als letztes setzen!
                        // Da hier mit Threads gearbeitet wird, ist das wichtig!

                        // Nchste Aktion vorbereiten:
                        if(escape[STATE] == ESCAPE_FRONT)
                        {
                            next_move_value = 110; // Hier sind es nun 110
                            // Wie bereits erwhnt, ist das nicht 100%ig genau,
                            // Abweichungen sind mglich!

                            next_direction = LEFT; // nchste Bewegungsrichtung
                        }
                        else if(escape[STATE] == ESCAPE_LEFT)
                        {
                            next_move_value = 70; // 70
                            next_direction = RIGHT;
                        }
                        else if(escape[STATE] == ESCAPE_RIGHT)
                        {
                            next_move_value = 70; // 70
                            next_direction = LEFT;
                        }
                        move_state = ROTATE; // nchster Zustand!
                    break;
                    case ROTATE:  // Rotieren ...
                        if(escape[ACTION] == NO_ACTION) // Letzte Bewegung abgeschlossen?
                        {
                            escape[SPEED_LEFT] = ESCAPE_SPEED_ROTATE;
                            escape[MOVE_DIR] = next_direction;
                            // Winkel in Grad einstellen:
                            escape[MOVE_VALUE_L] =  next_move_value & 0xFF;
                            escape[MOVE_VALUE_H] =  (next_move_value >> 8) & 0xFF;
                            escape[ACTION] = ACTION_ROTATE;  // Rotieren!
                            // Man sieht, dass der MOVE_VALUE Wert immer eine
                            // andere Bedeutung hat, je nachdem was in
                            // ACTION steht!

                            move_state = WAIT_END; // nchster Zustand!
                        }
                    break;
                    case WAIT_END: // Warten bis die Rotation abgeschlossen ist...
                        if(escape[ACTION] == NO_ACTION) // Letzte Bewegung abgeschlossen?
                        {
                            // Beide Zustnde zurcksetzen:
                            move_state = MOVE_BACKWARDS;
                            escape[STATE] = IDLE;
                        }
                    break;
              }
        }
        Thread_Delay(1);  // 10ms warten
    }
}


// -----------------------------------------------------------------------------
// Die Kontroll Routinen fr die Koordination der Verhalten:


/**
 * Dieser Funktion kann eines der Kommando Arrays der verschiedenen
 * Verhalten bergeben werden. Anhand der gesetzten Variablenwerte
 * werden dann die passenden Befehle an den Controller auf dem
 * Mainboard weitergeleitet (z.B. fahre mit 10cm/s  50cm geradeaus oder
 * setzte Bewegungsrichtung und Geschwindigkeit auf ...)
 */
void moveCommand(byte command[])
{
    byte speed_l, speed_r, dir;
    word move_val;

    speed_l = command[SPEED_LEFT];
    speed_r = command[SPEED_RIGHT];
    dir = command[MOVE_DIR];
    if(command[ACTION] == NO_ACTION) // Keine "Aktion", daher...
    {
        // ... einfach die Richtung und Geschwindigkeit setzen:
        changeDirection(dir);
        moveAtSpeed(speed_l,speed_r);
    }
	else // Ein Bewegungsablauf soll durchgefhrt werden...
    {
        move_val = command[MOVE_VALUE_L] + (command[MOVE_VALUE_H]<<8);
        if(move_val > 0) // Wenn der Wert gesetzt ist...
        {
            // ... dann je nach Inhalt des Felds ACTION eine
            // Bewegung mit diesem Wert auslsen:
            if(command[ACTION] == ACTION_ROTATE)
            {
                RP6_rotate(speed_l, dir, move_val, NON_BLOCKING);
            }
            else if(command[ACTION] == ACTION_MOVE)
            {
                RP6_move_mm(speed_l, dir, move_val, NON_BLOCKING);
            }
            // MOVE_VALUE zurcksetzen, damit die Aktion nur einmal
            // in Auftrag gegeben wird:
            command[MOVE_VALUE_L] = 0;
            command[MOVE_VALUE_H] = 0;
        }
        else if(movementComplete)  // Bewegungsvorgang abgeschlossen!
        {
            // ACTION Feld zurcksetzen:
            command[ACTION] = NO_ACTION;
        }
    }
}


/**
 * Dieser Thread realisiert die "Unterdrckung" von Verhaltensweisen
 * niedriger Prioritt durch solche mit hherer Prioritt. Im Prinzip sind
 * es nur ein paar if-else Abfragen hintereinander.
 * Ganz wichtig ist hierbei, dass KEIN Verhalten direkt auf Aktoren des
 * Roboters zugreift! Aktoren sind dabei z.B. Motoren, LEDs, LC-Display usw.
 * Alle Verhalten legen nur die ihrer Meinung nach passenden Befehle
 * in globalen Kommando-Arrays ab. Dieser Thread whlt die Befehle genau
 * EINES der Verhalten aus und leitet sie an den Roboter weiter.
 * Dabei wird stets das Verhalten mit der hchsten Prioritt gewhlt.
 *
 */
void behaviourController(void)
{
    while(true)
    {
        Thread_Lock(1); // Threadumschaltung sperren!
        // Das ist wichtig damit nicht whrend die Kommandos ausgefhrt
        // werden noch etwas an den Kommando Arrays gendert wird.

        // Hier werden die einzelnen Verhalten der Reihe nach
        // abgefragt.
        // Nochmal die Erluterung: Es kann nur EINES der Verhalten die
        // Kontrolle ber den Roboter bernehmen!
        // Wenn mehrere Verhalten aktiv sind, entscheidet die Prioritt
        // der Verhalten welches die Kontrolle bekommt.
        // Da hier ein if-else-if-else Konstrukt verwendet wird, hat das
        // oberste Verhalten auch die hchste Prioritt.
        // In diesem Fall hat "escape" also die hchste Prioritt, gefolgt
        // von "cruise". Ist gar kein Verhalten aktiv, werden einfach nur
        // die Motoren gestoppt. Das kann in diesem Beispiel aber gar nicht
        // eintreffen (knnte man allerdings tun, z.B. wenn die Akkus fast
        // leer sind)
        if(escape[STATE] != IDLE)      // Prioritt 2  - "escape"
        {
            moveCommand(escape);
        }
        else if(cruise[STATE] != IDLE) // Prioritt 1  - "cruise"
        {
            moveCommand(cruise);
        }
        else                           // Prioritt 0  - Nichts tun
        {
            stop();
        }

        Thread_Lock(0);  // Threadumschaltung wieder freigeben.
        Thread_Delay(5); // 50ms Pause
        // Die 50ms Pause verhindert zu schnelle Wechsel zwischen verschiedenen
        // Verhaltensweisen. Kann aber bei Bedarf angepasst werden!
    }
}


// -------------------------------------------------------------------------


void main(void)
{
	RP6_CCPRO_Init(); // Auf Startsignal warten!

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 Subsumption DEMO 2 ");
    newline();

    beep(100, 200, 300);

    showScreenLCD("Subsumption", "DEMO Program 2");
    AbsDelay(2000);
    clearLCD();

    // ACS auf mittlere Sendeleistung stellen:
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_MED);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT, true);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT_RQ, true);

    // Der Thread zur berprfung von Statusnderungen wird gestartet:
    Thread_Start(THREAD_CHECK_INT0,thread_checkINT0);

    // Die Threads der Verhaltensweisen starten:
    Thread_Start(BEHAVIOUR_CRUISE,behaviour_cruise);
    Thread_Start(BEHAVIOUR_ESCAPE,behaviour_escape);

    // Kontroll Thread fr die Verhalten (Arbiter) starten:
    Thread_Start(THREAD_BEHAVIOUR_CONTROLLER,behaviourController);

    setCursorPosLCD(1,0);
    printLCD("Moving...    :-)");

    while(true)
    {
        Thread_Delay(10);
    }
}


