/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * RP6 - Subsumption - Beispielprogramm 3
 * ----------------------------------------------------------------------------
 * Das Escape Verhalten ist noch nicht perfekt. Es kann noch weiter
 * verbessert werden! Es kann nmlich leicht passieren, dass der Roboter in eine
 * "Endlosschleife" gert!
 * Das heisst jetzt aber nicht, dass sich das Programm aufhngt!
 * Nein, er kann aber an bestimmten Hindernissen "hngen" bleiben und immer
 * hin und her schwenken bei jedem Ausweichmanver also genau in die
 * falsche Richtung drehen...
 * Schon bei einfachen Ecken eines Raumes (90 Winkel) kann sich diese
 * Situation einstellen!
 *
 * Das wollen wir in diesem Beispiel beheben, indem ein klein wenig
 * "Zufall" in das Bewegungsmuster eingebracht wird.
 * (es knnen natrlich noch weitere Dinge daran verbessert werden...
 * und neben der hier vorgeschlagenen Modifikation gibt es noch zig andere
 * Mglichkeiten, z.B. zufllige Rotationswinkel und Strecken oder messen
 * wie weit zwei Kollisionen auseinander lagen (ber die Drehgeber) und
 * dementsprechend versuchen "durch die Lcke dazwischen"
 * zu fahren - sofern vorhanden ...)
 *
 *
 * Der Rest des Programms wird nur wenig modifiziert, es kommen aber noch
 * zwei zustzliche Felder zum Kommando-Array hinzu. Nmlich um die LEDs
 * aus einem Verhalten heraus steuern zu knnen!
 * In einem Verhalten darf man ja keinesfalls direkt auf irgendwelche
 * Aktoren des Roboters zugreifen, und dazu zhlen auch die LEDs!
 * (das gilt natrlich nur, wenn mehrere verschiedene Verhalten auf eine
 * bestimmte Hardwarekomponente zugreifen - was in spteren Programmen
 * aber der Fall sein wird!)
 *
 * Die LEDs zeigen in diesem Beispiel wie schon so oft den Zustand der Motoren
 * und den Zustand der Verhaltensmuster an.
 *
 ******************************************************************************/

#include "../../RP6CCLib/RP6CCLib.cc"

// ###########################
#include "RP6MasterLib.cc"

// ###########################
#include "RP6_Sensorcheck.cc"


// Bei vielen Threads macht es sinn, die Thread Nummern durch lesbare
// Namen zu ersetzen:
#define THREAD_CHECK_INT0               1
#define THREAD_CHECK_BUMPERS            2
#define THREAD_BEHAVIOUR_CONTROLLER     3

// Namen fr Verhaltens Threads:
#define BEHAVIOUR_CRUISE   4
#define BEHAVIOUR_ESCAPE   5


// Neben den bereits bekannten Feldern fr das Kommando-Array:
#define STATE           0     // Zustand des Verhaltens
#define SPEED_LEFT      1     // Geschwindigkeitswert linker Motor
#define SPEED_RIGHT     2     // Geschwindigkeitswert rechter Motor
#define MOVE_DIR        3     // Bewegungsrichtung
#define ACTION          4     // Bewegungsaktion (Rotieren, bestimmte Distanz)
#define MOVE_VALUE_L    5     // Distanz oder Winkel, je nachdem ob es eine
#define MOVE_VALUE_H    6     // normale Bewegung oder eine Rotation ist
// kommen nun noch ein paar weitere hinzu:
#define LEDS            7     // LEDs auf dem Erweiterungsmodul
#define RP6_LEDS        8     // LEDs auf dem Mainboard

// Global definieren wieviele Felder unser Kommando-Array hat:
#define COMMAND_SIZE    9


// Fr das STATE Feld des Kommando Arrays gibt es fr jedes Verhalten
// zwei mgliche Zustnde:
#define IDLE    0
#define ACTIVE  1
// ABER: In den Verhalten werden meist noch weitere Zustnde hinzukommen!
// Fr den Kommando Interpreter ist aber nur wichtig, ob einer Verhalten
// Aktiv ist oder ob es gerade nichts zu melden hat (IDLE).



// Das neue ACTION Feld, kann folgende Zustnde annehmen:
#define NO_ACTION          0      // Keine Aktion
#define ACTION_ROTATE      1      // Rotieren
#define ACTION_MOVE        2      // Bestimmte Distanz abfahren



// -----------------------------------------------------------------------------
// Das CRUISE Verhalten:

// Das Kommando-Array fr das "Cruise" Verhalten:
byte cruise[COMMAND_SIZE];

// Bewegungsgeschwindigkeit fr "Cruise" Verhalten:
#define CRUISE_SPEED 80

/**
 * Dieses Verhalten ist sehr einfach. Der Roboter bewegt sich
 * schlicht und ergreifend einfach nur vorwrts.
 */
void behaviour_cruise(void)
{
    // Kommando-Array initialisieren:
    cruise[STATE] = ACTIVE; // Verhalten ist immer aktiv.
    cruise[SPEED_LEFT] = CRUISE_SPEED;
    cruise[SPEED_RIGHT] = CRUISE_SPEED;
    cruise[MOVE_DIR] = FWD;
    // Neu hinzugekommene Felder initialisieren (nichts tun...):
    cruise[ACTION] = NO_ACTION;
    cruise[MOVE_VALUE_L] =  0;
    cruise[MOVE_VALUE_H] =  0;
    cruise[LEDS] = LED1;  // Alles OK (grne LED) anzeigen!
    cruise[RP6_LEDS] = LED3 | LED6; // Vorwrts anzeigen...

    // In diesem Verhalten passiert sonst nichts weiter:
    while(true)
    {
        Thread_Delay(100);
    }
}


// -----------------------------------------------------------------------------
// Das ESCAPE Verhalten:

byte escape[COMMAND_SIZE];

// Geschwindigkeiten als Konstanten definieren:
#define ESCAPE_SPEED_BWD    70
#define ESCAPE_SPEED_ROTATE 60

#define ESCAPE_FRONT		1
#define ESCAPE_LEFT  		2
#define ESCAPE_RIGHT	    3

/**
 * Dieses Verhalten realisiert eine einfache Strategie um Hindernisse mithilfe
 * der Tastsensoren zu umfahren.
 *
 * Natrlich kollidiert der Roboter hiermit zunchst mit den Hindernissen,
 * das werden wir in einem spteren Beispielprogramm mithilfe des ACS
 * vermeiden. Allerdings ist das nicht immer zuverlssig, daher ist dieses
 * Verhalten auch damit noch sehr wichtig!
 *
 */
void behaviour_escape(void)
{
    // Kommando-Array initialisieren (nichts tun!):
    escape[STATE] = IDLE; // Verhalten ist inaktiv!
    escape[SPEED_LEFT] = 0;
    escape[SPEED_RIGHT] = 0;
    escape[MOVE_DIR] = FWD;
    escape[ACTION] = NO_ACTION;
    escape[MOVE_VALUE_L] =  0;
    escape[MOVE_VALUE_H] =  0;
    escape[LEDS] = 0;
    escape[RP6_LEDS] = 0;

    word next_move_value;
    byte next_direction;

    // Zustnde des Bewegungsablaufs:
    #define MOVE_BACKWARDS 0
    #define ROTATE         1
    #define WAIT_END       2
    byte move_state;

    // Zhlvariablen fr Zusammenste.
    byte bump_counter_left, bump_counter_right;
    bump_counter_left=0;
    bump_counter_right=0;
    // Damit wird festgelegt in welche Richtung sich der
    // Roboter drehen soll, falls es oft wechselseitige
    // Zusammenste gibt...

    while(true)
    {
        // Tastsensoren berprfen und dementsprechend
        // den Zustand wechseln:
        if(bumperLeft && bumperRight)   // Mitte!
        {
            // Neu hinzugekommene Abfrage um sicherzustellen das bei
            // gehaltenem Bumper nur einmal der Zustand gendert wird.
            // (das kann man alternativ auch ber die entsprechende Funktion
            // in der RP6_Sensorcheck.cc erreichen - die wird nur einmal
            // bei jeder nderung aufgerufen - aber hier wollen wir alles
            // in diesem Thread erledigen )
            if(escape[STATE] != ESCAPE_FRONT)
            {
                // Zhler fr beide Bumper erhhen:
                if(escape[STATE] != ESCAPE_LEFT)  // Verhindern das einer der Zhler
                    bump_counter_left++;          // doppelt hochgezhlt wird...
                if(escape[STATE] != ESCAPE_RIGHT)
                    bump_counter_right++;

                // Zustand auf "Frontalzusammensto" ndern:
                escape[STATE] = ESCAPE_FRONT;
                move_state = MOVE_BACKWARDS; // Zustand der Bewegung zurcksetzen!
                next_move_value = 300; // 300mm Rckwrts fahren...

                // LEDs auf dem Erweiterungsmodul setzen:
                escape[LEDS] = LED3 | LED4 | LED5;
            }
        }
        else if(bumperLeft)             // Links
        {
            // Falls nicht schon erkannt wurde das BEIDE
            // Tastsensoren aktiviert waren, kann der
            // Zustand auf "Zusammensto Links" gendert werden:
            if(escape[STATE] != ESCAPE_FRONT && escape[STATE] != ESCAPE_LEFT)
            {
                escape[STATE] = ESCAPE_LEFT;
                move_state = MOVE_BACKWARDS;
                next_move_value = 180; // 180mm

                // LEDs auf dem Erweiterungsmodul setzen:
                escape[LEDS] = LED3;
                // Zhler fr den linken Bumper erhhen:
                bump_counter_left++;
            }
            // Wenn dieser Zustand aktiviert wird,
            // ist der Roboter mit dem linken Bumper
            // gegen ein Hindernis gestoen. Das muss aber
            // noch nichts heissen, es kann sein, dass kurze
            // Zeit spter doch noch ESCAPE_FRONT aktiviert
            // wird. Der rechte Bumper knnte ein ganz klein
            // wenig spter aktiviert werden...
            // Mit der Abfrage oben if(escape[STATE] != ESCAPE_FRONT)
            // wird sichergestellt, dass auch der Zustand FRONT gehalten
            // wird wenn einer der Bumper wieder inaktiv wird.
        }
        else if(bumperRight)            // Rechts
        {
            // Ebenso fr die rechte Seite:
            if(escape[STATE] != ESCAPE_FRONT && escape[STATE] != ESCAPE_RIGHT)
            {
                escape[STATE] = ESCAPE_RIGHT;
                move_state = MOVE_BACKWARDS;
                next_move_value = 180; // 180mm

                // LEDs auf dem Erweiterungsmodul setzen:
                escape[LEDS] = LED5;
                // Zhler fr den rechten Bumper erhhen:
                bump_counter_right++;
            }
        }

        if(escape[STATE] != IDLE) // Verhalten ist aktiv? Dann Ausweichen...
        {
              // Zustandsautomat des Bewegungsablaufs:
              switch(move_state)
              {
                    case MOVE_BACKWARDS: // Rckwrts bewegen...
                        escape[SPEED_LEFT] = ESCAPE_SPEED_BWD;   // Geschwindigkeit setzen
                              // Das LINKE Geschwindigkeitsfeld gilt fr BEIDE Motoren wenn
                              // eine Bewegungsaktion durchgefhrt werden soll!
                        escape[MOVE_DIR] = BWD;                  // Rckwrts fahren

                        // Zu fahrende Strecke in Millimetern einstellen:
                        escape[MOVE_VALUE_L] =  next_move_value & 0xFF;
                        escape[MOVE_VALUE_H] =  (next_move_value >> 8) & 0xFF;
                        // Es gibt hier zwei Bytes, da grere Werte als 255
                        // mglich sein sollen! Man teilt einfach einen 16 Bit
                        // Wert in zwei 8 Bit Werte auf...

                        escape[ACTION] = ACTION_MOVE;  // Aktion: Bestimmte Distanz bewegen

                        escape[RP6_LEDS] = LED1 | LED4;

                        // Nchste Aktion vorbereiten:
                        if(escape[STATE] == ESCAPE_FRONT)
                        {
                            next_move_value = 110; // Hier sind es nun 110
                            // Wie bereits erwhnt, ist das nicht 100%ig genau,
                            // Abweichungen sind mglich!
                            if(bump_counter_left > bump_counter_right)
                            {
                                escape[RP6_LEDS] = escape[RP6_LEDS] | LED2;
                                next_direction = RIGHT; // nchste Bewegungsrichtung
                            }
                            else  if(bump_counter_left < bump_counter_right)
                            {
                                escape[RP6_LEDS] = escape[RP6_LEDS] | LED5;
                                next_direction = LEFT; // nchste Bewegungsrichtung
                            }
                            else
                            {
                                next_direction = LEFT; // nchste Bewegungsrichtung
                            }
                        }
                        else if(escape[STATE] == ESCAPE_LEFT)
                        {
                            // Zhlvariable RECHTS berprfen:
                            if(bump_counter_right > 2)
                            {
                                // Falls 3 oder mehr Zusammenste passiert sind,
                                // anders herum drehen als normal:
                                escape[RP6_LEDS] = escape[RP6_LEDS] | LED5;
                                bump_counter_left = 0;
                                bump_counter_right = 0;
                                next_move_value = 140; // 140
                                next_direction = LEFT;
                            }
                            else
                            {
                                // Normal nach rechts ausweichen:
                                escape[RP6_LEDS] = escape[RP6_LEDS] | LED2;
                                next_direction = RIGHT;
                                next_move_value = 60; // 60
                            }
                        }
                        else if(escape[STATE] == ESCAPE_RIGHT)
                        {
                            // Zhlvariable LINKS berprfen:
                            if(bump_counter_left > 2)
                            {
                                // Falls 3 oder mehr Zusammenste passiert sind,
                                // anders herum drehen als normal:
                                escape[RP6_LEDS] = escape[RP6_LEDS] | LED2;
                                bump_counter_left = 0;
                                bump_counter_right = 0;
                                next_move_value = 140; // 140
                                next_direction = RIGHT;
                            }
                            else
                            {
                                // Normal nach links ausweichen:
                                escape[RP6_LEDS] = escape[RP6_LEDS] | LED5;
                                next_move_value = 60; // 60
                                next_direction = LEFT;
                            }
                        }
                        move_state = ROTATE; // nchster Zustand!
                    break;
                    case ROTATE:  // Rotieren ...
                        if(escape[ACTION] == NO_ACTION) // Letzte Bewegung abgeschlossen?
                        {
                            escape[SPEED_LEFT] = ESCAPE_SPEED_ROTATE;
                            escape[MOVE_DIR] = next_direction;
                            // Winkel in Grad einstellen:
                            escape[MOVE_VALUE_L] =  next_move_value & 0xFF;
                            escape[MOVE_VALUE_H] =  (next_move_value >> 8) & 0xFF;
                            escape[ACTION] = ACTION_ROTATE;  // Rotieren!

                            if(next_direction == LEFT)
                                escape[RP6_LEDS] = LED3 | LED4;
                            else
                                escape[RP6_LEDS] = LED1 | LED6;
                            move_state = WAIT_END; // nchster Zustand!
                        }
                    break;
                    case WAIT_END: // Warten bis die Rotation abgeschlossen ist...
                        if(escape[ACTION] == NO_ACTION) // Letzte Bewegung abgeschlossen?
                        {
                            // Beide Zustnde zurcksetzen:
                            move_state = MOVE_BACKWARDS;
                            escape[STATE] = IDLE;
                            escape[RP6_LEDS] = 0;
                        }
                    break;
              }
        }
        Thread_Delay(1);  // 10ms warten
    }
}


// -----------------------------------------------------------------------------
// Die Kontroll Routinen fr die Koordination der Verhalten:


/**
 * Dieser Funktion kann eines der Kommando Arrays der verschiedenen
 * Verhalten bergeben werden. Anhand der gesetzten Variablenwerte
 * werden dann die passenden Befehle an den Controller auf dem
 * Mainboard weitergeleitet (z.B. fahre mit 10cm/s  50cm geradeaus oder
 * setzte Bewegungsrichtung und Geschwindigkeit auf ...)
 */
void moveCommand(byte command[])
{
    byte speed_l, speed_r, dir, leds, rp6_leds;
    word move_val;

    speed_l = command[SPEED_LEFT];
    speed_r = command[SPEED_RIGHT];
    dir = command[MOVE_DIR];
    leds = command[LEDS];
    rp6_leds = command[RP6_LEDS];

    setLEDs(leds);
    setRP6LEDs(rp6_leds);

    if(command[ACTION] == NO_ACTION) // Keine "Aktion", daher...
    {
        // ... einfach die Richtung und Geschwindigkeit setzen:
        changeDirection(dir);
        moveAtSpeed(speed_l,speed_r);
    }
	else // Ein Bewegungsablauf soll durchgefhrt werden...
    {
        move_val = command[MOVE_VALUE_L] + (command[MOVE_VALUE_H]<<8);
        if(move_val > 0) // Wenn der Wert gesetzt ist...
        {
            // ... dann je nach Inhalt des Felds ACTION eine
            // Bewegung mit diesem Wert auslsen:
            if(command[ACTION] == ACTION_ROTATE)
            {
                RP6_rotate(speed_l, dir, move_val, NON_BLOCKING);
            }
            else if(command[ACTION] == ACTION_MOVE)
            {
                RP6_move_mm(speed_l, dir, move_val, NON_BLOCKING);
            }
            // MOVE_VALUE zurcksetzen, damit die Aktion nur einmal
            // in Auftrag gegeben wird:
            command[MOVE_VALUE_L] = 0;
            command[MOVE_VALUE_H] = 0;
        }
        else if(movementComplete)  // Bewegungsvorgang abgeschlossen!
        {
            // ACTION Feld zurcksetzen:
            command[ACTION] = NO_ACTION;
        }
    }
}


/**
 * Dieser Thread realisiert die "Unterdrckung" von Verhaltensweisen
 * niedriger Prioritt durch solche mit hherer Prioritt. Im Prinzip sind
 * es nur ein paar if-else Abfragen hintereinander.
 * Ganz wichtig ist hierbei, dass KEIN Verhalten direkt auf Aktoren des
 * Roboters zugreift! Aktoren sind dabei z.B. Motoren, LEDs, LC-Display usw.
 * Alle Verhalten legen nur die ihrer Meinung nach passenden Befehle
 * in globalen Kommando-Arrays ab. Dieser Thread whlt die Befehle genau
 * EINES der Verhalten aus und leitet sie an den Roboter weiter.
 * Dabei wird stets das Verhalten mit der hchsten Prioritt gewhlt.
 *
 */
void behaviourController(void)
{
    while(true)
    {
        Thread_Lock(1); // Threadumschaltung sperren!
        // Das ist wichtig damit nicht whrend die Kommandos ausgefhrt
        // werden noch etwas an den Kommando Arrays gendert wird.

        // Hier werden die einzelnen Verhalten der Reihe nach
        // abgefragt.
        // Nochmal die Erluterung. Es kann nur EINES der Verhalten die
        // Kontrolle ber den Roboter bernehmen!
        // Wenn mehrere Verhalten aktiv sind, entscheidet die Prioritt
        // der Verhalten welches die Kontrolle bekommt.
        // Da hier ein if-else-if-else Konstrukt verwendet wird, hat das
        // oberste Verhalten auch die hchste Prioritt.
        // In diesem Fall hat "escape" also die hchste Prioritt, gefolgt
        // von "cruise". Ist gar kein Verhalten aktiv, werden einfach nur
        // die Motoren gestoppt. Das kann in diesem Beispiel aber gar nicht
        // eintreten (knnte man allerdings tun, z.B. wenn die Akkus fast
        // leer sind)
        if(escape[STATE] != IDLE)      // Prioritt 2  - "escape"
        {
            moveCommand(escape);
        }
        else if(cruise[STATE] != IDLE) // Prioritt 1  - "cruise"
        {
            moveCommand(cruise);
        }
        else                           // Prioritt 0  - Nichts tun
        {
            stop();
        }

        Thread_Lock(0);  // Threadumschaltung wieder freigeben.
        Thread_Delay(5); // 50ms Pause
        // Die 50ms Pause verhindert zu schnelle Wechsel zwischen verschiedenen
        // Verhaltensweisen. Kann aber bei Bedarf angepasst werden!
    }
}


// -------------------------------------------------------------------------


void main(void)
{
	RP6_CCPRO_Init(); // Auf Startsignal warten!

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 Subsumption DEMO 3 ");
    newline();

    beep(100, 200, 300);

    showScreenLCD("Subsumption", "DEMO Program 3");
    AbsDelay(2000);
    clearLCD();

    // ACS auf mittlere Sendeleistung stellen:
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_MED);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT, true);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT_RQ, true);

    // Der Thread zur berprfung von Statusnderungen wird gestartet:
    Thread_Start(THREAD_CHECK_INT0,thread_checkINT0);

    // Die Threads der Verhaltensweisen starten:
    Thread_Start(BEHAVIOUR_CRUISE,behaviour_cruise);
    Thread_Start(BEHAVIOUR_ESCAPE,behaviour_escape);

    // Kontroll Thread fr die Verhalten (Arbiter) starten:
    Thread_Start(THREAD_BEHAVIOUR_CONTROLLER,behaviourController);

    setCursorPosLCD(1,0);
    printLCD("Moving...    :-)");

    while(true)
    {
        Thread_Delay(100);
    }
}


