//------------------------------------------------------------------------------
// Unkommentierte Version von RP6_Behaviour3.cc

#include "../../RP6CCLib/RP6CCLib.cc"

#include "RP6MasterLib.cc"

#include "RP6_Sensorcheck.cc"


#define THREAD_CHECK_INT0               1
#define THREAD_CHECK_BUMPERS            2
#define THREAD_BEHAVIOUR_CONTROLLER     3

#define BEHAVIOUR_CRUISE   4
#define BEHAVIOUR_ESCAPE   5


#define STATE           0 
#define SPEED_LEFT      1 
#define SPEED_RIGHT     2
#define MOVE_DIR        3
#define ACTION          4
#define MOVE_VALUE_L    5
#define MOVE_VALUE_H    6 
#define LEDS            7
#define RP6_LEDS        8

#define COMMAND_SIZE    9

#define IDLE    0
#define ACTIVE  1

#define NO_ACTION          0 
#define ACTION_ROTATE      1
#define ACTION_MOVE        2 


byte cruise[COMMAND_SIZE];

#define CRUISE_SPEED 80

void behaviour_cruise(void)
{
    cruise[STATE] = ACTIVE;
    cruise[SPEED_LEFT] = CRUISE_SPEED;
    cruise[SPEED_RIGHT] = CRUISE_SPEED;
    cruise[MOVE_DIR] = FWD;
    cruise[ACTION] = NO_ACTION;
    cruise[MOVE_VALUE_L] =  0;
    cruise[MOVE_VALUE_H] =  0;
    cruise[LEDS] = LED1;
    cruise[RP6_LEDS] = LED3 | LED6;

    while(true)
    {
        Thread_Delay(100);
    }
}

byte escape[COMMAND_SIZE];

#define ESCAPE_SPEED_BWD    60
#define ESCAPE_SPEED_ROTATE 60

#define ESCAPE_FRONT		1
#define ESCAPE_LEFT  		2
#define ESCAPE_RIGHT	    3

void behaviour_escape(void)
{
    escape[STATE] = IDLE;
    escape[SPEED_LEFT] = 0;
    escape[SPEED_RIGHT] = 0;
    escape[MOVE_DIR] = FWD;
    escape[ACTION] = NO_ACTION;
    escape[MOVE_VALUE_L] =  0;
    escape[MOVE_VALUE_H] =  0;
    escape[LEDS] = 0;
    escape[RP6_LEDS] = 0;

    word next_move_value;
    byte next_direction;

    #define MOVE_BACKWARDS 0
    #define ROTATE         1
    #define WAIT_END       2
    byte move_state;

    byte bump_counter_left, bump_counter_right;
    bump_counter_left=0;
    bump_counter_right=0;

    while(true)
    {
        if(bumperLeft && bumperRight) 
        {
            if(escape[STATE] != ESCAPE_FRONT) 
            {
                if(escape[STATE] != ESCAPE_LEFT)
                    bump_counter_left++;
                if(escape[STATE] != ESCAPE_RIGHT)
                    bump_counter_right++;
                    
                escape[STATE] = ESCAPE_FRONT;
                move_state = MOVE_BACKWARDS;
                next_move_value = 300;
                
                escape[LEDS] = LED3 | LED4 | LED5;
            }
        }
        else if(bumperLeft)
        {
            if(escape[STATE] != ESCAPE_FRONT && escape[STATE] != ESCAPE_LEFT)
            {
                escape[STATE] = ESCAPE_LEFT;
                move_state = MOVE_BACKWARDS;
                next_move_value = 180;
                
                escape[LEDS] = LED3;
                bump_counter_left++;
            }
        }
        else if(bumperRight)
        {
            if(escape[STATE] != ESCAPE_FRONT && escape[STATE] != ESCAPE_RIGHT)
            {
                escape[STATE] = ESCAPE_RIGHT;
                move_state = MOVE_BACKWARDS;
                next_move_value = 180;
                
                escape[LEDS] = LED5;
                bump_counter_right++;
            }
        }

        if(escape[STATE] != IDLE)
        {
              switch(move_state)
              {
                    case MOVE_BACKWARDS:
                        escape[SPEED_LEFT] = ESCAPE_SPEED_BWD;
                        escape[MOVE_DIR] = BWD;
                        escape[MOVE_VALUE_L] =  next_move_value & 0xFF;
                        escape[MOVE_VALUE_H] =  (next_move_value >> 8) & 0xFF;
                        escape[ACTION] = ACTION_MOVE;

                        escape[RP6_LEDS] = LED1 | LED4;

                        if(escape[STATE] == ESCAPE_FRONT)
                        {
                            next_move_value = 100;
                            if(bump_counter_left > bump_counter_right)
                            {
                                escape[RP6_LEDS] = escape[RP6_LEDS] | LED2;
                                next_direction = RIGHT;
                            }
                            else  if(bump_counter_left < bump_counter_right)
                            {
                                escape[RP6_LEDS] = escape[RP6_LEDS] | LED5;
                                next_direction = LEFT;
                            }
                            else
                            {
                                next_direction = LEFT;
                            }
                        }
                        else if(escape[STATE] == ESCAPE_LEFT)
                        {
                            if(bump_counter_right > 2)
                            {
                                escape[RP6_LEDS] = escape[RP6_LEDS] | LED5;
                                bump_counter_left = 0;
                                bump_counter_right = 0;
                                next_move_value = 140;
                                next_direction = LEFT;
                            }
                            else
                            {
                                escape[RP6_LEDS] = escape[RP6_LEDS] | LED2;
                                next_direction = RIGHT;
                                next_move_value = 60;
                            }
                        }
                        else if(escape[STATE] == ESCAPE_RIGHT)
                        {
                            if(bump_counter_left > 2)
                            {
                                escape[RP6_LEDS] = escape[RP6_LEDS] | LED2;
                                bump_counter_left = 0;
                                bump_counter_right = 0;
                                next_move_value = 140;
                                next_direction = RIGHT;
                            }
                            else
                            {
                                escape[RP6_LEDS] = escape[RP6_LEDS] | LED5;
                                next_move_value = 60;
                                next_direction = LEFT;
                            }
                        }
                        move_state = ROTATE;
                    break;
                    case ROTATE:
                        if(escape[ACTION] == NO_ACTION)
                        {
                            escape[SPEED_LEFT] = ESCAPE_SPEED_ROTATE;
                            escape[MOVE_DIR] = next_direction;
                           
                            escape[MOVE_VALUE_L] =  next_move_value & 0xFF;
                            escape[MOVE_VALUE_H] =  (next_move_value >> 8) & 0xFF;
                            escape[ACTION] = ACTION_ROTATE; 

                            if(next_direction == LEFT)
                                escape[RP6_LEDS] = LED3 | LED4;
                            else
                                escape[RP6_LEDS] = LED1 | LED6;
                            move_state = WAIT_END;
                        }
                    break;
                    case WAIT_END:
                        if(escape[ACTION] == NO_ACTION)
                        {
                            move_state = MOVE_BACKWARDS;
                            escape[STATE] = IDLE;
                            escape[RP6_LEDS] = 0;
                        }
                    break;
              }
        }
        Thread_Delay(1);
    }
}

void moveCommand(byte command[])
{
    byte speed_l, speed_r, dir, leds, rp6_leds;
    word move_val;

    speed_l = command[SPEED_LEFT];
    speed_r = command[SPEED_RIGHT];
    dir = command[MOVE_DIR];
    leds = command[LEDS];
    rp6_leds = command[RP6_LEDS];

    setLEDs(leds);
    setRP6LEDs(rp6_leds);

    if(command[ACTION] == NO_ACTION)
    {
        changeDirection(dir);
        moveAtSpeed(speed_l,speed_r);
    }
	else
    {
        move_val = command[MOVE_VALUE_L] + (command[MOVE_VALUE_H]<<8);
        if(move_val > 0)
        {
            if(command[ACTION] == ACTION_ROTATE)
            {
                RP6_rotate(speed_l, dir, move_val, NON_BLOCKING);
            }
            else if(command[ACTION] == ACTION_MOVE)
            {
                RP6_move_mm(speed_l, dir, move_val, NON_BLOCKING);
            }
            command[MOVE_VALUE_L] = 0;
            command[MOVE_VALUE_H] = 0;
        }
        else if(movementComplete)
        {
            command[ACTION] = NO_ACTION;
        }
    }
}

void behaviourController(void)
{
    while(true)
    {
        Thread_Lock(1);
        if(escape[STATE] != IDLE)
        {
            moveCommand(escape);
        }
        else if(cruise[STATE] != IDLE)
        {
            moveCommand(cruise);
        }
        else                          
        {
            stop();
        }
        Thread_Lock(0);
        Thread_Delay(5);
    }
}


void main(void)
{
	RP6_CCPRO_Init();

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 Subsumption DEMO 3 ");
    newline();

    beep(100, 200, 300);

    showScreenLCD("Subsumption", "DEMO Program 3");
    AbsDelay(2000);
    clearLCD();

    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_MED);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT, true);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT_RQ, true);

    Thread_Start(THREAD_CHECK_INT0,thread_checkINT0);

    Thread_Start(BEHAVIOUR_CRUISE,behaviour_cruise);
    Thread_Start(BEHAVIOUR_ESCAPE,behaviour_escape);

    Thread_Start(THREAD_BEHAVIOUR_CONTROLLER,behaviourController);

    setCursorPosLCD(1,0);
    printLCD("Moving...    :-)");

    while(true)
    {
        Thread_Delay(100);
    }
}
