/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * RP6 - Subsumption - Beispielprogramm 4
 * ----------------------------------------------------------------------------
 * Achtung:
 * Dieses Programm ist schon ziemlich komplex und richtet sich eher an
 * Fortgeschrittene! Anfnger sollten mit einfacheren Programmen beginnen!
 *
 * ---------------
 *
 * Mit den Bumpern kann der Roboter zwar detektieren, dass er mit einem
 * Hindernis zusammengestoen ist, jedoch nicht bereits vorher ausweichen.
 * Aber dafr gibt es ja das ACS (Anti Collision System).
 * Weitere Infos zum ACS selbst finden Sie in der ausfhrlichen RP6 Anleitung.
 *
 * Das ACS Verhalten ist sogar etwas einfacher zu implementieren als
 * das der Bumper, denn hier muss nicht zurckgesetzt und rotiert werden.
 * Es reicht bei detektierten Objekten links oder rechts eine Kurve zu fahren
 * und wenn ein Hindernis in der Mitte erkannt wurde, wird einfach auf der
 * Stelle rotiert.
 *
 * Sollte das ACS Hindernisse nur unzureichend erkennen, knnen Sie versuchen
 * die Sendeleistung zu erhhren indem Sie die Zeile:
 * RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_MED);
 * weiter unten im Programm gegen:
 * RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_HIGH);
 * ersetzen!
 *
 * Das in diesem Programm implementierte Verhalten kann durchaus noch
 * verbessert werden und ist nicht perfekt!
 * Dies ist allerdings das letzte Beispielprogramm zum Verhaltensbasierten
 * Roboter. Jetzt knnen Sie eigene zustzliche Verhaltensweisen hinzufgen,
 * z.B. knnen noch die IR Sensoren zum verfolgen von Lichtquellen
 * eingesetzt werden. Auch neue Sensoren kann man dem Roboter hinzufgen,
 * damit er sich besser in seiner Umgebung zurechtfinden kann.
 * (zustzliche IR Sensoren, Ultraschallsensoren, Kompass usw. )
 *
 * Neben dem neu hinzugekommenen ACS Verhalten, wurde nun auch die Ausgabe
 * auf dem LC-Display verbessert. Es wird nun stets das gerade aktive Verhalten
 * dargestellt. Die LCD Ausgaben werden von einem eigenen Thread gesteuert,
 * denn aus verschiedenen Threads darf nicht gleichzeitig auf das Display
 * geschrieben werden ...
 *
 * --------------------------------------------------------------------------
 * Hinweis: Wie bereits an anderer Stelle erwhnt, versteht sich dieses
 * Programm genau wie alle anderen nur als Ausgangspunkt. An vielen Stellen
 * knnte es noch erweitert und verbessert werden!
 ******************************************************************************/

#include "../../RP6CCLib/RP6CCLib.cc"

// ###########################
#include "RP6MasterLib.cc"

// ###########################
#include "RP6_Sensorcheck.cc"


// Bei vielen Threads macht es sinn, die Thread Nummern durch lesbare
// Namen zu ersetzen:
#define THREAD_CHECK_INT0               1
#define THREAD_CHECK_BUMPERS            2
#define THREAD_BEHAVIOUR_CONTROLLER     3
#define THREAD_DISPLAY_CONTROLLER       4

// Namen fr Verhaltens Threads:
#define BEHAVIOUR_CRUISE   5
#define BEHAVIOUR_ESCAPE   6
#define BEHAVIOUR_AVOID    7

// Felder fr das Kommando-Array:
#define STATE           0     // Zustand des Verhaltens
#define SPEED_LEFT      1     // Geschwindigkeitswert linker Motor
#define SPEED_RIGHT     2     // Geschwindigkeitswert rechter Motor
#define MOVE_DIR        3     // Bewegungsrichtung
#define ACTION          4     // Bewegungsaktion (Rotieren, bestimmte Distanz)
#define MOVE_VALUE_L    5     // Distanz oder Winkel, je nachdem ob es eine
#define MOVE_VALUE_H    6     // normale Bewegung oder eine Rotation ist
#define LEDS            7     // LEDs auf dem Erweiterungsmodul
#define RP6_LEDS        8     // LEDs auf dem Mainboard

// Global definieren wieviele Felder unser Kommando-Array hat:
#define COMMAND_SIZE    9


// Fr das STATE Feld des Kommando Arrays gibt es fr jedes Verhalten
// zwei mgliche Zustnde:
#define IDLE    0
#define ACTIVE  1
// ABER: In den Verhalten werden meist noch weitere Zustnde hinzukommen!
// Fr den Kommando Interpreter ist aber nur wichtig, ob einer Verhalten
// Aktiv ist oder ob es gerade nichts zu melden hat (IDLE).



// Das neue ACTION Feld, kann folgende Zustnde annehmen:
#define NO_ACTION          0      // Keine Aktion
#define ACTION_ROTATE      1      // Rotieren
#define ACTION_MOVE        2      // Bestimmte Distanz abfahren



// -----------------------------------------------------------------------------
// Das CRUISE Verhalten:

// Das Kommando-Array fr das "Cruise" Verhalten:
byte cruise[COMMAND_SIZE];

// Bewegungsgeschwindigkeit fr "Cruise" Verhalten:
#define CRUISE_SPEED 90

/**
 * Dieses Verhalten ist sehr einfach. Der Roboter bewegt sich
 * schlicht und ergreifend einfach nur vorwrts.
 */
void behaviour_cruise(void)
{
    // Kommando-Array initialisieren:
    cruise[STATE] = ACTIVE; // Verhalten ist immer aktiv.
    cruise[SPEED_LEFT] = CRUISE_SPEED;
    cruise[SPEED_RIGHT] = CRUISE_SPEED;
    cruise[MOVE_DIR] = FWD;
    // Neu hinzugekommene Felder initialisieren (nichts tun...):
    cruise[ACTION] = NO_ACTION;
    cruise[MOVE_VALUE_L] =  0;
    cruise[MOVE_VALUE_H] =  0;
    cruise[LEDS] = LED1;
    cruise[RP6_LEDS] = LED1 | LED4;

    byte runLight, dir;
    runLight = 1;
    dir = 0;

    // Kleines Lauflicht ausfhren solange Cruise aktiv ist:
    while(true)
    {
		cruise[LEDS] = runLight;

		if(runLight >= 16) {
            dir = 1;
        }
		else if (runLight <= 1) {
			dir = 0;
        }

		if(dir == 0)
			runLight = runLight << 1;
		else
			runLight = runLight >> 1;

        Thread_Delay(10);
    }
}


// -----------------------------------------------------------------------------
// Das ESCAPE Verhalten:

byte escape[COMMAND_SIZE];

// Geschwindigkeiten als Konstanten definieren:
#define ESCAPE_SPEED_BWD    70
#define ESCAPE_SPEED_ROTATE 60


#define ESCAPE_FRONT		1
#define ESCAPE_LEFT  		2
#define ESCAPE_RIGHT	    3

/**
 * Dieses Verhalten realisiert eine einfache Strategie um Hindernisse mithilfe
 * der Tastsensoren zu umfahren.
 *
 * Natrlich kollidiert der Roboter hiermit zunchst mit den Hindernissen,
 * das werden wir in einem spteren Beispielprogramm mithilfe des ACS
 * vermeiden. Allerdings ist das nicht immer zuverlssig, daher ist dieses
 * Verhalten auch damit noch sehr wichtig!
 *
 */
void behaviour_escape(void)
{
    // Kommando-Array initialisieren (nichts tun!):
    escape[STATE] = IDLE; // Verhalten ist inaktiv!
    escape[SPEED_LEFT] = 0;
    escape[SPEED_RIGHT] = 0;
    escape[MOVE_DIR] = FWD;
    escape[ACTION] = NO_ACTION;
    escape[MOVE_VALUE_L] =  0;
    escape[MOVE_VALUE_H] =  0;
    escape[LEDS] = 0;
    escape[RP6_LEDS] = 0;

    word next_move_value;
    byte next_direction;

    // Zustnde des Bewegungsablaufs:
    #define MOVE_BACKWARDS 0
    #define ROTATE         1
    #define WAIT_END       2
    byte move_state;

    // Zhlvariablen fr Zusammenste.
    byte bump_counter_left, bump_counter_right;
    bump_counter_left=0;
    bump_counter_right=0;
    // Damit wird festgelegt in welche Richtung sich der
    // Roboter drehen soll, falls es oft wechselseitige
    // zusammenste gibt...

    while(true)
    {
        // Tastsensoren berprfen und dementsprechend
        // den Zustand wechseln:
        if(bumperLeft && bumperRight)   // Mitte!
        {
            // Neu hinzugekommene Abfrage um sicherzustellen das bei
            // gehaltenem Bumper nur einmal der Zustand gendert wird.
            // (das kann man alternativ auch ber die entsprechende Funktion
            // in der RP6_Sensorcheck.cc erreichen - die wird nur einmal
            // bei jeder nderung aufgerufen - aber hier wollen wir alles
            // in diesem Thread erledigen )
            if(escape[STATE] != ESCAPE_FRONT)
            {
                // Zhler fr den beide Bumper erhhen:
                if(escape[STATE] != ESCAPE_LEFT)  // Verhindern das einer der Zhler
                    bump_counter_left++;          // doppelt hochgezhlt wird...
                if(escape[STATE] != ESCAPE_RIGHT)
                    bump_counter_right++;

                // Zustand auf "Frontalzusammensto" ndern:
                escape[STATE] = ESCAPE_FRONT;
                move_state = MOVE_BACKWARDS; // Zustand der Bewegung zurcksetzen!
                next_move_value = 300; // 300mm Rckwrts fahren...

                // LEDs auf dem Erweiterungsmodul setzen:
                escape[LEDS] = LED3 | LED4 | LED5;
            }
        }
        else if(bumperLeft)             // Links
        {
            // Falls nicht schon erkannt wurde das BEIDE
            // Tastsensoren aktiviert waren, kann der
            // Zustand auf "Zusammensto Links" gendert werden:
            if(escape[STATE] != ESCAPE_FRONT && escape[STATE] != ESCAPE_LEFT)
            {
                escape[STATE] = ESCAPE_LEFT;
                move_state = MOVE_BACKWARDS;
                next_move_value = 180; // 180mm

                // LEDs auf dem Erweiterungsmodul setzen:
                escape[LEDS] = LED3;
                // Zhler fr den linken Bumper erhhen:
                bump_counter_left++;
            }
            // Wenn dieser Zustand aktiviert wird,
            // ist der Roboter mit dem linken Bumper
            // gegen ein Hindernis gestoen. Das muss aber
            // noch nichts heissen, es kann sein, dass kurze
            // Zeit spter doch noch ESCAPE_FRONT aktiviert
            // wird. Der rechte Bumper knnte ein ganz klein
            // wenig spter aktiviert werden...
            // Mit der Abfrage oben if(escape[STATE] != ESCAPE_FRONT)
            // wird sichergestellt, dass auch der Zustand FRONT gehalten
            // wird wenn einer der Bumper wieder inaktiv wird.
        }
        else if(bumperRight)            // Rechts
        {
            // Ebenso fr die rechte Seite:
            if(escape[STATE] != ESCAPE_FRONT && escape[STATE] != ESCAPE_RIGHT)
            {
                escape[STATE] = ESCAPE_RIGHT;
                move_state = MOVE_BACKWARDS;
                next_move_value = 180; // 180mm

                // LEDs auf dem Erweiterungsmodul setzen:
                escape[LEDS] = LED5;
                // Zhler fr den rechten Bumper erhhen:
                bump_counter_right++;
            }
        }

        if(escape[STATE] != IDLE) // Verhalten ist aktiv? Dann Ausweichen...
        {
              // Zustandsautomat des Bewegungsablaufs:
              switch(move_state)
              {
                  case MOVE_BACKWARDS: // Rckwrts bewegen...
                      escape[SPEED_LEFT] = ESCAPE_SPEED_BWD;   // Geschwindigkeit setzen
                            // Das LINKE Geschwindigkeitsfeld gilt fr BEIDE Motoren wenn
                            // eine Bewegungsaktion durchgefhrt werden soll!
                      escape[MOVE_DIR] = BWD;                  // Rckwrts fahren

                      // Zu fahrende Strecke in Millimetern einstellen:
                      escape[MOVE_VALUE_L] =  next_move_value & 0xFF;
                      escape[MOVE_VALUE_H] =  (next_move_value >> 8) & 0xFF;
                      // Es gibt hier zwei Bytes, da grere Werte als 255
                      // mglich sein sollen!
                      escape[ACTION] = ACTION_MOVE;  // Aktion: Bestimmte Distanz bewegen

                      escape[RP6_LEDS] = LED1 | LED4;

                      // Nchste Aktion vorbereiten:
                      if(escape[STATE] == ESCAPE_FRONT)
                      {
                          next_move_value = 110; // Hier sind es nun 110
                          // Wie bereits erwhnt, ist das nicht 100%ig genau,
                          // Abweichungen sind mglich!
                          printInteger(bump_counter_right);
                          if(bump_counter_left > bump_counter_right)
                          {
                              escape[RP6_LEDS] = escape[RP6_LEDS] | LED2;
                              next_direction = RIGHT; // nchste Bewegungsrichtung
                          }
                          else  if(bump_counter_left < bump_counter_right)
                          {
                              escape[RP6_LEDS] = escape[RP6_LEDS] | LED5;
                              next_direction = LEFT; // nchste Bewegungsrichtung
                          }
                          else
                          {
                              next_direction = LEFT; // nchste Bewegungsrichtung
                          }
                      }
                      else if(escape[STATE] == ESCAPE_LEFT)
                      {
                          // Zhlvariable RECHTS berprfen:
                          if(bump_counter_right > 2)
                          {
                              // Falls 3 oder mehr Zusammenste passiert sind,
                              // anders herum drehen als normal:
                              escape[RP6_LEDS] = escape[RP6_LEDS] | LED5;
                              bump_counter_left = 0;
                              bump_counter_right = 0;
                              next_move_value = 140; // 140
                              next_direction = LEFT;
                          }
                          else
                          {
                              escape[RP6_LEDS] = escape[RP6_LEDS] | LED2;
                              next_direction = RIGHT;
                              next_move_value = 60; // 60
                          }
                      }
                      else if(escape[STATE] == ESCAPE_RIGHT)
                      {
                          // Zhlvariable LINKS berprfen:
                          if(bump_counter_left > 2)
                          {
                              // Falls 3 oder mehr Zusammenste passiert sind,
                              // anders herum drehen als normal:
                              escape[RP6_LEDS] = escape[RP6_LEDS] | LED2;
                              bump_counter_left = 0;
                              bump_counter_right = 0;
                              next_move_value = 140; // 140
                              next_direction = RIGHT;
                          }
                          else
                          {
                              escape[RP6_LEDS] = escape[RP6_LEDS] | LED5;
                              next_move_value = 60; // 60
                              next_direction = LEFT;
                          }
                      }
                      move_state = ROTATE; // nchster Zustand!
                  break;
                  case ROTATE:  // Rotieren ...
                      if(escape[ACTION] == NO_ACTION) // Letzte Bewegung abgeschlossen?
                      {
                          escape[SPEED_LEFT] = ESCAPE_SPEED_ROTATE;
                          escape[MOVE_DIR] = next_direction;
                          // Winkel in Grad einstellen:
                          escape[MOVE_VALUE_L] =  next_move_value & 0xFF;
                          escape[MOVE_VALUE_H] =  (next_move_value >> 8) & 0xFF;
                          escape[ACTION] = ACTION_ROTATE;  // Rotieren!

                          if(next_direction == LEFT)
                              escape[RP6_LEDS] = LED3 | LED4;
                          else
                              escape[RP6_LEDS] = LED1 | LED6;
                          move_state = WAIT_END; // nchster Zustand!
                      }
                  break;
                  case WAIT_END: // Warten bis die Rotation abgeschlossen ist...
                      if(escape[ACTION] == NO_ACTION) // Letzte Bewegung abgeschlossen?
                      {
                          // Beide Zustnde zurcksetzen:
                          move_state = MOVE_BACKWARDS;
                          escape[STATE] = IDLE;
                          escape[RP6_LEDS] = 0;
                      }
                  break;
              }
        }
        Thread_Delay(1);  // 10ms warten
    }
}


// -----------------------------------------------------------------------------
// Das AVOID Verhalten:

// Das Kommando-Array fr das "Avoid" Verhalten:
byte avoid[COMMAND_SIZE];

// Geschwindigkeiten als Konstanten definieren:
#define AVOID_SPEED_NORMAL    85
#define AVOID_SPEED_CURVE     30
#define AVOID_SPEED_ROTATE    60


#define AVOID_FRONT  1
#define AVOID_LEFT   2
#define AVOID_RIGHT  3

/**
 * Das AVOID Verhalten versucht Hindernissen auszuweichen.
 * Es ist grundstzlich sehr hnlich aufgebaut wie ESCAPE,
 * aber etwas bersichtlicher da nicht zurckgesetzt
 * werden muss.
 */
void behaviour_avoid(void)
{
    avoid[STATE] = IDLE;
    avoid[SPEED_LEFT] = 0;
    avoid[SPEED_RIGHT] = 0;
    avoid[MOVE_DIR] = FWD;
    avoid[ACTION] = NO_ACTION;
    avoid[MOVE_VALUE_L] =  0;
    avoid[MOVE_VALUE_H] =  0;
    avoid[LEDS] = LED1;
    avoid[RP6_LEDS] = LED1 | LED4;

    // Zustnde des Bewegungsablaufs:
    #define AVOID_CURVE         0
    #define AVOID_ROTATE        1
    #define AVOID_WAIT_END      2
    byte move_state;
    byte move_timer;
    byte next_move;
    next_move = LEFT;

    while(true)
    {
        // Die Auswertung funktioniert fast genauso wie bei
        // den Bumpern:
        if(obstacleLeft && obstacleRight)   // Mitte!
        {
            avoid[LEDS] = LED3 | LED4 | LED5;
            avoid[RP6_LEDS] = LED2 | LED3 | LED5 | LED6;
            move_state = AVOID_ROTATE;
            avoid[STATE] = AVOID_FRONT;
        }
        else if(obstacleLeft)             // Links
        {
            if(avoid[STATE] != AVOID_FRONT)
            {
                avoid[STATE] = AVOID_LEFT;
                avoid[LEDS] = LED3;
                move_state = AVOID_CURVE;
                next_move = RIGHT;
            }
            avoid[RP6_LEDS] = LED1 | LED6;
        }
        else if(obstacleRight)            // Rechts
        {
            if(avoid[STATE] != AVOID_FRONT)
            {
                avoid[STATE] = AVOID_RIGHT;
                avoid[LEDS] = LED5;
                move_state = AVOID_CURVE;
                next_move = LEFT;
            }
            avoid[RP6_LEDS] = LED3 | LED4;
        }
        else
            avoid[RP6_LEDS] = LED1 | LED4;

        if(avoid[STATE] != IDLE) // Verhalten ist aktiv? Dann Ausweichen...
        {
             // Das Ausweichen funktioniert von der Struktur her (Zustandsautomat)
             // sehr hnlich wie bei den Bumpern, ist aber
             // etwas bersichtlicher:
             switch(move_state)
             {
                  case AVOID_CURVE:  // Kurve fahren:
                      if(avoid[STATE] == AVOID_RIGHT)
                      {
                          avoid[SPEED_LEFT] = AVOID_SPEED_CURVE;
                          avoid[SPEED_RIGHT] = AVOID_SPEED_NORMAL;
                      }
                      else if(avoid[STATE] == AVOID_LEFT)
                      {
                          avoid[SPEED_LEFT] = AVOID_SPEED_NORMAL;
                          avoid[SPEED_RIGHT] = AVOID_SPEED_CURVE;
                      }
                      avoid[MOVE_DIR] = FWD;
                      move_timer = 25;
                      move_state = AVOID_WAIT_END;
                  break;
                  case ROTATE: // Auf der Stelle rotieren:
                      avoid[SPEED_LEFT] = AVOID_SPEED_ROTATE;
                      avoid[SPEED_RIGHT] = AVOID_SPEED_ROTATE;
                      avoid[MOVE_DIR] = next_move;
                      move_timer = 100;
                      move_state = AVOID_WAIT_END;
                  break;
                  case AVOID_WAIT_END: // Zeitgesteuert warten:
                      if(move_timer < 1)
                      {
                          move_state = MOVE_BACKWARDS;
                          avoid[STATE] = IDLE;
                          avoid[RP6_LEDS] = 0;
                      }
                      else
                          move_timer--;
                  break;
             }
        }

        Thread_Delay(1);
    }
}

// -----------------------------------------------------------------------------
// Die Kontroll Routinen fr die Koordination der Verhalten:

byte last_behaviour;
byte last_command[COMMAND_SIZE];

/**
 * Dieser Funktion kann eines der Kommando Arrays der verschiedenen
 * Verhalten bergeben werden. Anhand der gesetzten Variablenwerte
 * werden dann die passenden Befehle an den Controller auf dem
 * Mainboard weitergeleitet (z.B. fahre mit 10cm/s  50cm geradeaus oder
 * setzte Bewegungsrichtung und Geschwindigkeit auf ...)
 *
 * Die Funktion wurde hier etwas verbessert - sie kann nun erkennen ob
 * sich ein Befehl gendert hat oder ob noch immer alles beim alten
 * geblieben ist. So knnen doppelte Transfers ber den I2C Bus
 * vermieden werden (dauert sehr viel lnger als das berprfen ob
 * sich etwas gendert hat)
 *
 */
void moveCommand(byte command[], byte behaviour_number)
{
    // Lokale Variablen deklarieren:
    byte action, speed_l, speed_r, dir;
    word move_val;
    byte leds, rp6_leds;
    char temp_str[32];

    byte i;
    byte changed;
    changed = false;

    if(behaviour_number != last_behaviour) { // Verhalten hat sich gendert?
        last_behaviour = behaviour_number;
        changed = true;
    }
    // Hat sich der Befehl (des gleich gebliebenen Verhaltens) gendert?
    for(i = 0; i < COMMAND_SIZE; i++)
    {
        if(last_command[i] != command[i])
            changed = true;
        last_command[i] = command[i];
    }

    if(changed) // Falls sich etwas verndert hat...
    {
        // Auf dem Display darstellen, welches Verhalten gerade aktiv ist.
        // Mit dem neuen Display update Thread schreibt man einfach den
        // gewnschten Text in die entsprechende Zeilenvariable und anschlieend
        // setzt man update_display auf true.
        if(behaviour_number == BEHAVIOUR_AVOID)
        {
            display_line2 = "[AVOID : ";
            // Genauer darstellen in welchem Zustand sich Avoid befindet:
            if(avoid[STATE] == AVOID_FRONT) {
                temp_str = "FRONT]";
            } else if(avoid[STATE] == AVOID_LEFT) {
                temp_str = "LEFT]";
            } else if(avoid[STATE] == AVOID_RIGHT) {
                temp_str = "RIGHT]";
            } else {
                temp_str = "]";
            }
            // Zusatzinfo an den String anfgen:
            Str_Copy(display_line2,temp_str,STR_APPEND);
        }
        else if(behaviour_number == BEHAVIOUR_ESCAPE)
        {
            display_line2 = "[ESCAPE : ";

            // Genauer darstellen in welchem Zustand sich Escape befindet:
            if(escape[STATE] == ESCAPE_FRONT) {
                temp_str = "FRONT]";
            } else if(escape[STATE] == ESCAPE_LEFT) {
                temp_str = "LEFT]";
            } else if(escape[STATE] == ESCAPE_RIGHT) {
                temp_str = "RIGHT]";
            } else {
                temp_str = "]";
            }
            // Zusatzinfo an den String anfgen:
            Str_Copy(display_line2,temp_str,STR_APPEND);
        }
        else if(behaviour_number == BEHAVIOUR_CRUISE)
        {
            display_line2 = "[CRUISE]";
        }
        else
        {
            display_line2 = "[ --- ]";
        }
        update_display = true; // Display Inhalt erneuern...

        // ... genderte Daten an den Controller auf dem Mainboard senden.
        action = command[ACTION];
        speed_l = command[SPEED_LEFT];
        speed_r = command[SPEED_RIGHT];
        dir = command[MOVE_DIR];
        leds = command[LEDS];
        rp6_leds = command[RP6_LEDS];

        setLEDs(leds);
        setRP6LEDs(rp6_leds);

        if(command[ACTION] == NO_ACTION) // Keine "Aktion", daher...
        {
            // ... einfach die Richtung und Geschwindigkeit setzen:
            changeDirection(dir);
            moveAtSpeed(speed_l, speed_r);
        }
        else // Ein Bewegungsablauf soll durchgefhrt werden...
        {
            move_val = command[MOVE_VALUE_L] + (command[MOVE_VALUE_H]<<8);
            if(move_val > 0) // Wenn der Wert gesetzt ist...
            {
                // ... dann je nach Inhalt des Felds ACTION eine
                // Bewegung mit diesem Wert auslsen:
                if(command[ACTION] == ACTION_ROTATE)
                {
                    RP6_rotate(speed_l, dir, move_val, NON_BLOCKING);
                }
                else if(command[ACTION] == ACTION_MOVE)
                {
                    RP6_move_mm(speed_l, dir, move_val, NON_BLOCKING);
                }
                // MOVE_VALUE zurcksetzen, damit die Aktion nur einmal
                // in Auftrag gegeben wird:
                command[MOVE_VALUE_L] = 0;
                command[MOVE_VALUE_H] = 0;
            }
            else if(movementComplete)  // Bewegungsvorgang abgeschlossen?
            {
                // ACTION Feld zurcksetzen:
                command[ACTION] = NO_ACTION;
            }
        }
    }
    else if(movementComplete)  // Bewegungsvorgang abgeschlossen?
    {
        // ACTION Feld zurcksetzen:
        command[ACTION] = NO_ACTION;
    }
}


// Kommando das ausgefhrt wird, falls KEIN Verhalten
// aktiv ist:
byte stop_cmd[COMMAND_SIZE];


/**
 * Dieser Thread realisiert die "Unterdrckung" von Verhaltensweisen
 * niedriger Prioritt durch solche mit hherer Prioritt. Im Prinzip sind
 * es nur ein paar if-else Abfragen hintereinander.
 * Ganz wichtig ist hierbei, dass KEIN Verhalten direkt auf Aktoren des
 * Roboters zugreift! Aktoren sind dabei z.B. Motoren, LEDs, LC-Display usw.
 * Alle Verhalten legen nur die ihrer Meinung nach passenden Befehle
 * in Globalen Kommando-Array ab. Dieser Thread whlt die Befehle genau
 * EINES der Verhalten aus und leitet sie an den Roboter weiter.
 * Dabei wird stets das Verhalten mit der hchsten Prioritt gewhlt.
 *
 */
void behaviourController(void)
{
    // Stop Kommando initialisieren:
    stop_cmd[STATE] = ACTIVE;
    stop_cmd[SPEED_LEFT] = 0;
    stop_cmd[SPEED_RIGHT] = 0;
    stop_cmd[MOVE_DIR] = FWD;
    stop_cmd[ACTION] = NO_ACTION;
    stop_cmd[MOVE_VALUE_L] = 0;
    stop_cmd[MOVE_VALUE_H] = 0;
    stop_cmd[LEDS] = LED1;
    stop_cmd[RP6_LEDS] = 0;

    while(true)
    {
        Thread_Lock(1); // Threadumschaltung sperren!
        // Das ist wichtig damit nicht whrend die Kommandos ausgefhrt
        // werden noch etwas an den Kommando Arrays gendert wird.

        // Hier werden die einzelnen Verhalten der Reihe nach
        // abgefragt.
        // Nochmal die Erluterung. Es kann nur EINES der Verhalten die
        // Kontrolle ber den Roboter bernehmen!
        // Wenn mehrere Verhalten aktiv sind, entscheidet die Prioritt
        // der Verhalten welches die Kontrolle bekommt.
        // Da hier ein if-else-if-else Konstrukt verwendet wird, hat das
        // oberste Verhalten auch die hchste Prioritt.
        // In diesem Fall hat "escape" also die hchste Prioritt, gefolgt
        // von "cruise". Ist gar kein Verhalten aktiv, werden einfach nur
        // die Motoren gestoppt. Das kann in diesem Beispiel aber gar nicht
        // eintreffen (knnte man allerdings tun, z.B. wenn die Akkus fast
        // leer sind)
        if(escape[STATE] != IDLE)      // Prioritt 3  - "escape"
        {
            moveCommand(escape, BEHAVIOUR_ESCAPE);
        }
        else if(avoid[STATE] != IDLE)      // Prioritt 2  - "avoid"
        {
            moveCommand(avoid, BEHAVIOUR_AVOID);
        }
        else if(cruise[STATE] != IDLE) // Prioritt 1  - "cruise"
        {
            moveCommand(cruise, BEHAVIOUR_CRUISE);
        }
        else                           // Prioritt 0  - Nichts tun
        {
            moveCommand(stop_cmd, 0);
            stop();
        }

        Thread_Lock(0);  // Threadumschaltung wieder freigeben.
        Thread_Delay(3); // 30ms Pause
        // Die 30ms Pause verhindert zu schnelle Wechsel zwischen verschiedenen
        // Verhaltensweisen. Kann aber bei Bedarf angepasst werden!
    }
}



// -----------------------------------------------------------------------------
// Display

char display_line1[17];
char display_line2[17];
byte update_display;
byte update_sensor_displays;

/**
 * Wie bereits in vorangegangen Beispielprogrammen gezeigt, kann
 * nur aus EINEM Thread auf das Display zugegriffen werden, da es sonst
 * leicht zu fehlerhafter Darstellung kommen kann (muss nicht immer passieren,
 * kann aber in ungnstigen Situationen auftreten).
 * Da wir hier viele verschiedene Threads haben, deren Daten wir gerne auf dem
 * einen Display darstellen wrden, bietet es sich an, diese Daten in einem
 * eigenen Thread zum Display zu senden. Nebenbei sollte ohnehin immer nur das
 * gerade aktive Verhalten seine Daten auf dem Display darstellen knnen.
 *
 * Was hier genau dargestellt wird, kann natrlich beliebig angepasst werden.
 * Aber alle Ausgaben sollte in diesem einen Thread erfolgen und in keinem
 * anderen!
 * In diesem Beispiel werden Zustand von Bumpern und ACS als kleine Symbole
 * in Displayzeile 1 dargestellt und in Zeile 2 kann ein beliebiger Text
 * dargestellt werden.
 * Auch die Heartbeat Anzeige wird nun von hier aus auf dem Display dargestellt.
 *
 * Um Rechenzeit zu sparen, wird der Inhalt nur bei nderungen erneuert und
 * nur die notwendigen Regionen gelscht. Das vorher zu ermitteln geht schneller
 * als einfach alles auf das Display zu schreiben, da die bertragung zum
 * Display nur recht langsam erfolgen kann.
 *
 * ACHTUNG: Diese Routine geht von einem 16x2 Zeichen Display aus. Wenn Sie
 * ein anderes Display Format verwenden, mssen hier einige Dinge angepasst
 * werden.
 */
void displayController(void)
{
    byte last_heartbeat, len;
    while(true)
    {
        // Text auf dem Display erneuern, falls notwendig:
        if(update_display)
        {
            update_display = false;

            // len = Str_Len(display_line1);
            //    clearPosLCD(0, len, 16-len);
            //    setCursorPosLCD(0,0);
            //    writeStringLCD(display_line1);
            // Wurde auskommentiert da es in diesem Beispiel nicht bentigt
            // wird und so Rechenzeit gespart werden kann.

            len = Str_Len(display_line2);
            clearPosLCD(1, len, 16-len);
            setCursorPosLCD(1,0);
            writeStringLCD(display_line2);
        }

        // Heartbeat Display:
        if(last_heartbeat != heartbeat)
        {
            last_heartbeat = heartbeat;
            if(heartbeat)
            {
                setCursorPosLCD(0,11);
                printLCD("*");
            }
            else
            {
                clearPosLCD(0, 11, 1);
            }
        }

        if(update_sensor_displays)
        {
            // Status von Bumpern UND dem ACS in Zeile 1 vom LCD anzeigen:
            if(obstacleRight) {  // Hindernis rechts!
                setCursorPosLCD(0,15);
                printLCD("o");
            } else {               // Kein Hindernis.
                setCursorPosLCD(0,15);
                printLCD(" ");
            }
            if(obstacleLeft) {  // Hindernis links!
                setCursorPosLCD(0,0);
                printLCD("o");
            } else {               // Kein Hindernis.
                setCursorPosLCD(0,0);
                printLCD(" ");
            }
            if(obstacleRight && obstacleLeft) {   // Mitte!
                setCursorPosLCD(0,7);
                printLCD("O");
            } else {
                setCursorPosLCD(0,7);
                printLCD(" ");
            }

            // Bumper:
            if(bumperRight) {  // Bumper rechts!
                setCursorPosLCD(0,14);
                printLCD("-");
            } else {               // Kein Zusammensto.
                setCursorPosLCD(0,14);
                printLCD(" ");
            }
            if(bumperLeft) {     // Bumper links!
                setCursorPosLCD(0,1);
                printLCD("-");
            } else {               // Kein Zusammensto.
                setCursorPosLCD(0,1);
                printLCD(" ");
            }
            if(bumperLeft && bumperRight) {   // Beide Bumper
                setCursorPosLCD(0,6);
                printLCD("-");
                setCursorPosLCD(0,8);
                printLCD("-");
            } else {
                setCursorPosLCD(0,6);
                printLCD(" ");
                setCursorPosLCD(0,8);
                printLCD(" ");
            }
        }
        Thread_Delay(5); // 50ms Pause
    }
}

// -------------------------------------------------------------------------

void main(void)
{
	RP6_CCPRO_Init(); // Auf Startsignal warten!

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 Subsumption DEMO 4  ");
    newline();

    beep(100, 200, 300);

    showScreenLCD("Subsumption", "DEMO Program 4");
    AbsDelay(2000);
    clearLCD();

    // ACS auf mittlere Sendeleistung stellen:
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_MED);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT, true);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT_RQ, true);

    // Der Thread zur berprfung von Statusnderungen wird gestartet:
    Thread_Start(THREAD_CHECK_INT0,thread_checkINT0);

    // Die Threads der Verhaltensweisen starten:
    Thread_Start(BEHAVIOUR_CRUISE,behaviour_cruise);
    Thread_Start(BEHAVIOUR_ESCAPE,behaviour_escape);
    Thread_Start(BEHAVIOUR_AVOID,behaviour_avoid);

    // Kontroll Thread fr die Verhalten (Arbiter) starten:
    Thread_Start(THREAD_BEHAVIOUR_CONTROLLER,behaviourController);

    // Neue Display Verwaltung starten:
    Thread_Start(THREAD_DISPLAY_CONTROLLER,displayController);

    while(true)
    {
        Thread_Delay(100);
    }
}
