//------------------------------------------------------------------------------
// Unkommentierte Version von RP6_Behaviour4.cc

#include "../../RP6CCLib/RP6CCLib.cc"
#include "RP6MasterLib.cc"
#include "RP6_Sensorcheck.cc"

#define THREAD_CHECK_INT0               1
#define THREAD_CHECK_BUMPERS            2
#define THREAD_BEHAVIOUR_CONTROLLER     3
#define THREAD_DISPLAY_CONTROLLER       4

#define BEHAVIOUR_CRUISE   5
#define BEHAVIOUR_ESCAPE   6
#define BEHAVIOUR_AVOID    7

#define STATE           0 
#define SPEED_LEFT      1
#define SPEED_RIGHT     2
#define MOVE_DIR        3
#define ACTION          4
#define MOVE_VALUE_L    5
#define MOVE_VALUE_H    6
#define LEDS            7
#define RP6_LEDS        8

#define COMMAND_SIZE    9

#define IDLE    0
#define ACTIVE  1


#define NO_ACTION          0
#define ACTION_ROTATE      1
#define ACTION_MOVE        2



byte cruise[COMMAND_SIZE];

#define CRUISE_SPEED 90

void behaviour_cruise(void)
{
    cruise[STATE] = ACTIVE;
    cruise[SPEED_LEFT] = CRUISE_SPEED;
    cruise[SPEED_RIGHT] = CRUISE_SPEED;
    cruise[MOVE_DIR] = FWD;
    cruise[ACTION] = NO_ACTION;
    cruise[MOVE_VALUE_L] =  0;
    cruise[MOVE_VALUE_H] =  0;
    cruise[LEDS] = LED1;
    cruise[RP6_LEDS] = LED1 | LED4;

    byte runLight, dir;
    runLight = 1;
    dir = 0;

    while(true)
    {
		cruise[LEDS] = runLight;

		if(runLight >= 16) {
            dir = 1;
        }
		else if (runLight <= 1) {
			dir = 0;
        }

		if(dir == 0)
			runLight = runLight << 1;
		else
			runLight = runLight >> 1;

        Thread_Delay(10);
    }
}



byte escape[COMMAND_SIZE];

#define ESCAPE_SPEED_BWD    70
#define ESCAPE_SPEED_ROTATE 60

#define ESCAPE_FRONT		1
#define ESCAPE_LEFT  		2
#define ESCAPE_RIGHT	    3

void behaviour_escape(void)
{
    
    escape[STATE] = IDLE;
    escape[SPEED_LEFT] = 0;
    escape[SPEED_RIGHT] = 0;
    escape[MOVE_DIR] = FWD;
    escape[ACTION] = NO_ACTION;
    escape[MOVE_VALUE_L] =  0;
    escape[MOVE_VALUE_H] =  0;
    escape[LEDS] = 0;
    escape[RP6_LEDS] = 0;

    word next_move_value;
    byte next_direction;

    #define MOVE_BACKWARDS 0
    #define ROTATE         1
    #define WAIT_END       2
    byte move_state;

    byte bump_counter_left, bump_counter_right;
    bump_counter_left=0;
    bump_counter_right=0;

    while(true)
    {
        if(bumperLeft && bumperRight)
        {
            if(escape[STATE] != ESCAPE_FRONT)
            {
                if(escape[STATE] != ESCAPE_LEFT)
                    bump_counter_left++;
                if(escape[STATE] != ESCAPE_RIGHT)
                    bump_counter_right++;

                escape[STATE] = ESCAPE_FRONT;
                move_state = MOVE_BACKWARDS;
                next_move_value = 300;

                escape[LEDS] = LED3 | LED4 | LED5;
            }
        }
        else if(bumperLeft)
        {
            if(escape[STATE] != ESCAPE_FRONT && escape[STATE] != ESCAPE_LEFT)
            {
                escape[STATE] = ESCAPE_LEFT;
                move_state = MOVE_BACKWARDS;
                next_move_value = 180;

                escape[LEDS] = LED3;
                bump_counter_left++;
            }
        }
        else if(bumperRight)
        {
            if(escape[STATE] != ESCAPE_FRONT && escape[STATE] != ESCAPE_RIGHT)
            {
                escape[STATE] = ESCAPE_RIGHT;
                move_state = MOVE_BACKWARDS;
                next_move_value = 180;

                escape[LEDS] = LED5;
                bump_counter_right++;
            }
        }

        if(escape[STATE] != IDLE)
        {
              switch(move_state)
              {
                  case MOVE_BACKWARDS:
                      escape[SPEED_LEFT] = ESCAPE_SPEED_BWD; 
                      escape[MOVE_DIR] = BWD;
                      escape[MOVE_VALUE_L] =  next_move_value & 0xFF;
                      escape[MOVE_VALUE_H] =  (next_move_value >> 8) & 0xFF;
                      escape[ACTION] = ACTION_MOVE;
                      escape[RP6_LEDS] = LED1 | LED4;

                      if(escape[STATE] == ESCAPE_FRONT)
                      {
                          next_move_value = 100;
                          printInteger(bump_counter_right);
                          if(bump_counter_left > bump_counter_right)
                          {
                              escape[RP6_LEDS] = escape[RP6_LEDS] | LED2;
                              next_direction = RIGHT;
                          }
                          else  if(bump_counter_left < bump_counter_right)
                          {
                              escape[RP6_LEDS] = escape[RP6_LEDS] | LED5;
                              next_direction = LEFT;
                          }
                          else
                          {
                              next_direction = LEFT;
                          }
                      }
                      else if(escape[STATE] == ESCAPE_LEFT)
                      {
                          if(bump_counter_right > 2)
                          {
                              escape[RP6_LEDS] = escape[RP6_LEDS] | LED5;
                              bump_counter_left = 0;
                              bump_counter_right = 0;
                              next_move_value = 140;
                              next_direction = LEFT;
                          }
                          else
                          {
                              escape[RP6_LEDS] = escape[RP6_LEDS] | LED2;
                              next_direction = RIGHT;
                              next_move_value = 60;
                          }
                      }
                      else if(escape[STATE] == ESCAPE_RIGHT)
                      {
                          if(bump_counter_left > 2)
                          {
                              escape[RP6_LEDS] = escape[RP6_LEDS] | LED2;
                              bump_counter_left = 0;
                              bump_counter_right = 0;
                              next_move_value = 140;
                              next_direction = RIGHT;
                          }
                          else
                          {
                              escape[RP6_LEDS] = escape[RP6_LEDS] | LED5;
                              next_move_value = 60;
                              next_direction = LEFT;
                          }
                      }
                      move_state = ROTATE;
                  break;
                  case ROTATE:
                      if(escape[ACTION] == NO_ACTION)
                      {
                          escape[SPEED_LEFT] = ESCAPE_SPEED_ROTATE;
                          escape[MOVE_DIR] = next_direction;
                          escape[MOVE_VALUE_L] =  next_move_value & 0xFF;
                          escape[MOVE_VALUE_H] =  (next_move_value >> 8) & 0xFF;
                          escape[ACTION] = ACTION_ROTATE;  // Rotieren!

                          if(next_direction == LEFT)
                              escape[RP6_LEDS] = LED3 | LED4;
                          else
                              escape[RP6_LEDS] = LED1 | LED6;
                          move_state = WAIT_END;
                      }
                  break;
                  case WAIT_END: 
                      if(escape[ACTION] == NO_ACTION)
                      {
                          move_state = MOVE_BACKWARDS;
                          escape[STATE] = IDLE;
                          escape[RP6_LEDS] = 0;
                      }
                  break;
              }
        }
        Thread_Delay(1);
    }
}



byte avoid[COMMAND_SIZE];

#define AVOID_SPEED_NORMAL    85
#define AVOID_SPEED_CURVE     30
#define AVOID_SPEED_ROTATE    60

#define AVOID_FRONT  1
#define AVOID_LEFT   2
#define AVOID_RIGHT  3

void behaviour_avoid(void)
{
    avoid[STATE] = IDLE;
    avoid[SPEED_LEFT] = 0;
    avoid[SPEED_RIGHT] = 0;
    avoid[MOVE_DIR] = FWD;
    avoid[ACTION] = NO_ACTION;
    avoid[MOVE_VALUE_L] =  0;
    avoid[MOVE_VALUE_H] =  0;
    avoid[LEDS] = LED1;
    avoid[RP6_LEDS] = LED1 | LED4;

    #define AVOID_CURVE         0
    #define AVOID_ROTATE        1
    #define AVOID_WAIT_END      2
    byte move_state;
    byte move_timer;
    byte next_move;
    next_move = LEFT;

    while(true)
    {
        if(obstacleLeft && obstacleRight)
        {
            avoid[LEDS] = LED3 | LED4 | LED5;
            avoid[RP6_LEDS] = LED2 | LED3 | LED5 | LED6;
            move_state = AVOID_ROTATE;
            avoid[STATE] = AVOID_FRONT;
        }
        else if(obstacleLeft)
        {
            if(avoid[STATE] != AVOID_FRONT)
            {
                avoid[STATE] = AVOID_LEFT;
                avoid[LEDS] = LED3;
                move_state = AVOID_CURVE;
                next_move = RIGHT;
            }
            avoid[RP6_LEDS] = LED1 | LED6;
        }
        else if(obstacleRight)
        {
            if(avoid[STATE] != AVOID_FRONT)
            {
                avoid[STATE] = AVOID_RIGHT;
                avoid[LEDS] = LED5;
                move_state = AVOID_CURVE;
                next_move = LEFT;
            }
            avoid[RP6_LEDS] = LED3 | LED4;
        }
        else
            avoid[RP6_LEDS] = LED1 | LED4;

        if(avoid[STATE] != IDLE)
        {
             switch(move_state)
             {
                  case AVOID_CURVE:
                      if(avoid[STATE] == AVOID_RIGHT)
                      {
                          avoid[SPEED_LEFT] = AVOID_SPEED_CURVE;
                          avoid[SPEED_RIGHT] = AVOID_SPEED_NORMAL;
                      }
                      else if(avoid[STATE] == AVOID_LEFT)
                      {
                          avoid[SPEED_LEFT] = AVOID_SPEED_NORMAL;
                          avoid[SPEED_RIGHT] = AVOID_SPEED_CURVE;
                      }
                      avoid[MOVE_DIR] = FWD;
                      move_timer = 25;
                      move_state = AVOID_WAIT_END;
                  break;
                  case ROTATE:
                      avoid[SPEED_LEFT] = AVOID_SPEED_ROTATE;
                      avoid[SPEED_RIGHT] = AVOID_SPEED_ROTATE;
                      avoid[MOVE_DIR] = next_move;
                      move_timer = 100;
                      move_state = AVOID_WAIT_END;
                  break;
                  case AVOID_WAIT_END:
                      if(move_timer < 1)
                      {
                          move_state = MOVE_BACKWARDS;
                          avoid[STATE] = IDLE;
                          avoid[RP6_LEDS] = 0;
                      }
                      else
                          move_timer--;
                  break;
             }
        }

        Thread_Delay(1);
    }
}


byte last_command[COMMAND_SIZE];
byte last_behaviour;

void moveCommand(byte command[], byte behaviour_number)
{
    byte action, speed_l, speed_r, dir;
    word move_val;
    byte leds, rp6_leds;
    char temp_str[32];

    byte i;
    byte changed;
    changed = false;

    if(behaviour_number != last_behaviour) {
        last_behaviour = behaviour_number;
        changed = true;
    }

    for(i = 0; i < COMMAND_SIZE; i++)
    {
        if(last_command[i] != command[i])
            changed = true;
        last_command[i] = command[i];
    }

    if(changed)
    {
        if(behaviour_number == BEHAVIOUR_AVOID)
        {
            display_line2 = "[AVOID : ";
            if(avoid[STATE] == AVOID_FRONT) {
                temp_str = "FRONT]";
            } else if(avoid[STATE] == AVOID_LEFT) {
                temp_str = "LEFT]";
            } else if(avoid[STATE] == AVOID_RIGHT) {
                temp_str = "RIGHT]";
            } else {
                temp_str = "]";
            }
            Str_Copy(display_line2,temp_str,STR_APPEND);
        }
        else if(behaviour_number == BEHAVIOUR_ESCAPE)
        {
            display_line2 = "[ESCAPE : ";
            if(escape[STATE] == ESCAPE_FRONT) {
                temp_str = "FRONT]";
            } else if(escape[STATE] == ESCAPE_LEFT) {
                temp_str = "LEFT]";
            } else if(escape[STATE] == ESCAPE_RIGHT) {
                temp_str = "RIGHT]";
            } else {
                temp_str = "]";
            }
            Str_Copy(display_line2,temp_str,STR_APPEND);
        }
        else if(behaviour_number == BEHAVIOUR_CRUISE)
        {
            display_line2 = "[CRUISE]";
        }
        else
        {
            display_line2 = "[ --- ]";
        }
        update_display = true;

        action = command[ACTION];
        speed_l = command[SPEED_LEFT];
        speed_r = command[SPEED_RIGHT];
        dir = command[MOVE_DIR];
        leds = command[LEDS];
        rp6_leds = command[RP6_LEDS];

        setLEDs(leds);
        setRP6LEDs(rp6_leds);

        if(command[ACTION] == NO_ACTION)
        {
            changeDirection(dir);
            moveAtSpeed(speed_l, speed_r);
        }
        else
        {
            move_val = command[MOVE_VALUE_L] + (command[MOVE_VALUE_H]<<8);
            if(move_val > 0)
            {
                if(command[ACTION] == ACTION_ROTATE)
                {
                    RP6_rotate(speed_l, dir, move_val, NON_BLOCKING);
                }
                else if(command[ACTION] == ACTION_MOVE)
                {
                    RP6_move_mm(speed_l, dir, move_val, NON_BLOCKING);
                }
                command[MOVE_VALUE_L] = 0;
                command[MOVE_VALUE_H] = 0;
            }
            else if(movementComplete)
            {
                command[ACTION] = NO_ACTION;
            }
        }
    }
    else if(movementComplete)
    {
        command[ACTION] = NO_ACTION;
    }
}


void behaviourController(void)
{
    while(true)
    {
        Thread_Lock(1); 
        if(escape[STATE] != IDLE)
        {
            moveCommand(escape, BEHAVIOUR_ESCAPE);
        }
        else if(avoid[STATE] != IDLE)
        {
            moveCommand(avoid, BEHAVIOUR_AVOID);
        }
        else if(cruise[STATE] != IDLE)
        {
            moveCommand(cruise, BEHAVIOUR_CRUISE);
        }
        else
        {
            stop();
        }

        Thread_Lock(0);
        Thread_Delay(5);
    }
}


char display_line1[17];
char display_line2[17];
byte update_display;

void displayController(void)
{
    byte last_heartbeat, len;
    while(true)
    {
        if(update_display)
        {
            update_display = false;

            // len = Str_Len(display_line1);
            //    clearPosLCD(0, len, 16-len);
            //    setCursorPosLCD(0,0);
            //    writeStringLCD(display_line1);
            // Wurde auskommentiert da es in diesem Beispiel nicht bentigt
            // wird und so Rechenzeit gespart werden kann.

            len = Str_Len(display_line2);
            clearPosLCD(1, len, 16-len);
            setCursorPosLCD(1,0);
            writeStringLCD(display_line2);
        }

        if(last_heartbeat != heartbeat)
        {
            last_heartbeat = heartbeat;
            if(heartbeat)
            {
                setCursorPosLCD(0,11);
                printLCD("*");
            }
            else
            {
                clearPosLCD(0, 11, 1);
            }
        }

        if(obstacleRight) {
            setCursorPosLCD(0,15);
            printLCD("o");
        } else {
            setCursorPosLCD(0,15);
            printLCD(" ");
        }
        if(obstacleLeft) {
            setCursorPosLCD(0,0);
            printLCD("o");
        } else {
            setCursorPosLCD(0,0);
            printLCD(" ");
        }
        if(obstacleRight && obstacleLeft) {
            setCursorPosLCD(0,7);
            printLCD("O");
        } else {
            setCursorPosLCD(0,7);
            printLCD(" ");
        }

        if(bumperRight) {
            setCursorPosLCD(0,14);
            printLCD("-");
        } else {
            setCursorPosLCD(0,14);
            printLCD(" ");
        }
        if(bumperLeft) { 
            setCursorPosLCD(0,1);
            printLCD("-");
        } else {
            setCursorPosLCD(0,1);
            printLCD(" ");
        }
        if(bumperLeft && bumperRight) {
            setCursorPosLCD(0,6);
            printLCD("-");
            setCursorPosLCD(0,8);
            printLCD("-");
        } else {
            setCursorPosLCD(0,6);
            printLCD(" ");
            setCursorPosLCD(0,8);
            printLCD(" ");
        }

        Thread_Delay(5);
    }
}


void main(void)
{
	RP6_CCPRO_Init();

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 Subsumption DEMO 4  ");
    newline();

    beep(100, 200, 300);

    showScreenLCD("Subsumption", "DEMO Program 4");
    AbsDelay(2000);
    clearLCD();

    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_ACS_POWER, ACS_PWR_MED);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT, true);
    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SET_WDT_RQ, true);

    Thread_Start(THREAD_CHECK_INT0,thread_checkINT0);
    Thread_Start(BEHAVIOUR_CRUISE,behaviour_cruise);
    Thread_Start(BEHAVIOUR_ESCAPE,behaviour_escape);
    Thread_Start(BEHAVIOUR_AVOID,behaviour_avoid);
    Thread_Start(THREAD_BEHAVIOUR_CONTROLLER,behaviourController);
    Thread_Start(THREAD_DISPLAY_CONTROLLER,displayController);

    while(true)
    {
        Thread_Delay(100);
    }
}
