/*******************************************************************************
 * RP6 C-Control PRO M128 Erweiterungsmodul
 * ----------------------------------------------------------------------------
 * I2C Bus Beispielprogramm
 * ----------------------------------------------------------------------------
 * In diese Datei wurde die Sensorauswertung ausgelagert. So bleibt
 * das Programm insgesamt bersichtlicher.
 *
 *
 * ACHTUNG: Der Programmteil wurde fr die neue Display Update
 * Routine angepasst! Die alten Ausgaben wurde gelscht.
 ******************************************************************************/

//------------------------------------------------------------------------------
//

#ifndef __SENSORCHECK__     // Alles nur EINMAL einbinden!
#define __SENSORCHECK__
// Diese zwei Zeilen und die letzte in dieser Datei sind wichtig, sonst
// funktioniert das bersetzen nicht! Auch bei eigenen Erweiterungen MUSS das
// am Anfang stehen!

/**
 * Wenn sich der Zustand des ACS verndert hat, wird diese Funktion
 * aufgerufen. Was hier genau gemacht wird, hngt dann von der Anwendung ab.
 */
void handle_ACS(void)
{
    beep_t(200, 1, 1);
    update_sensor_displays = true;
}

/**
 * Genauso wie bei der ACS Funktion, wird handle_Bumpers aufgerufen sobald
 * sich der Zustand der Tastsensoren vom RP6 gendert hat.
 */
void handle_Bumpers(void)
{
    beep_t(100, 1, 1);
    update_sensor_displays = true;
}

/**
 * Bei niedriger Akkuspannung wird diese Funktion aufgerufen
 */
void handle_BatLow(void)
{
    print("BAT LOW:");
    printInteger(adcBat);
    println(" ");
}

/**
 * Sobald Bewegungsablufe abgeschlossen sind, oder bei einer
 * Fehlerabschaltung aufgrund blockierter Motoren wird diese Funktion
 * aufgerufen.
 */
void handle_DriveSystemChange(void)
{
}


/**
 * Der RP6 kann mit dem IR Empfnger vom ACS auch RC5 Codes von Fernbedienungen
 * empfangen. Sobald ein Tastencode empfangen wurde, wird diese Funktion
 * aufgerufen.
 */
void handle_RC5_reception(void)
{
    beep_t((RC5_data+5)*15, 2, 2);
    print(" RC5: ");
    printInteger(RC5_toggle);   // Togglebit (Taste gehalten
    print(":");                 // oder mehrfach gedrckt?)
    printInteger(RC5_adr);      // Adresse
    print(":");
    printInteger(RC5_data);     // Tastencode
    println(" ");
}

/**
 * Damit der Roboter nicht ausser Kontrolle gert, wenn das CCPRO oder andere
 * Host Controller abstrzen (oder die Kommunikation aus sonstigen Grnden
 * gestrt ist), kann ein Watchdog Timer aktiviert werden, der
 * alle 250ms eine Anfrage an den Host sendet. Antwortet dieser nicht innerhalb
 * von 3 Sekunden auf diese Anfrage, werden die Motoren gestoppt und der Roboter
 * muss zurckgesetzt werden bevor er weiterfahren kann.
 *
 * Man kann diese periodischen Anfragen auch gut verwenden um zu signalisieren
 * dass das Programm noch luft, indem man eine LED blinken lsst, oder wie
 * in diesem Beispielprogramm ein Zeichen auf dem LC-Display ("Heartbeat").
 */
byte heartbeat;
void handle_WDT_RQ(void)
{
    if(heartbeat)
    {
        heartbeat = false;
    }
    else
    {
        heartbeat = true;
    }
}

/**
 * Dies ist die etwas aufgerumte Interrupt Behandlungsroutine. Die eigentliche
 * Auswertung der Daten erfolgt in den einzelnen Funktionen oben. Diese
 * werden hier bei Bedarf aufgerufen.
 */
void thread_checkINT0(void)
{
    while(true) {
        if(Port_ReadBit(PORT_PE5_INT)) { // Hat der Controller auf dem Mainboard neue Daten?
            // Die drei Status Register auslesen:
            RP6_readRegisters(RP6_BASE_ADR, 0, messageBuf, 3);

            // Wir berprfen, ob die Status Bits sich verndert haben,
            // also verknpfen wir den alten und neuen Wert per Exclusiv Oder
            // um nur die genderten Bits auswerten zu mssen.
            byte compare;
            compare = messageBuf[0] ^ interrupt_status;

            interrupt_status = messageBuf[0];

            // ------------------------------------
            // Zunchst prfen wir ob sich die ACS Status Bits verndert haben.
            // Falls nicht, hat irgend etwas anderes den Interrupt ausgelst und
            // wir knnen mit den anderen Abfragen weitermachen.
            if(compare & MASK_ACS)
            {
                 // Die ACS Status Variablen setzen:
                 obstacleLeft = messageBuf[0] & 0x20;
                 obstacleRight = messageBuf[0] & 0x40;

                 // ACS in getrennter Funktion weiterbehandeln:
                 handle_ACS();
            }

            // ------------------------------------
            // Abfragen ob sich der Zustand der Bumper gendert hat, falls
            // ja, Text auf der seriellen Schnittstelle ausgeben:
            if(compare & MASK_BUMPER)
            {
                 bumperLeft = messageBuf[0] & 0x2;
                 bumperRight = messageBuf[0] & 0x4;

                 handle_Bumpers();
            }

            // ------------------------------------
            // berprfen ob ein RC5 Datenpaket mit dem IRCOMM empfangen
            // worden ist - falls ja auf der seriellen ausgeben:
            if(interrupt_status & MASK_RC5)
            {
                RP6_readRegisters(RP6_BASE_ADR, I2C_REG_RC5_ADR, messageBuf, 2);
                RC5_toggle = messageBuf[0] >> 5;
                RC5_adr = messageBuf[0] & 0x1F;
                RC5_data = messageBuf[1];

                handle_RC5_reception();
            }

            // ------------------------------------
            // berprfen ob die Batteriespannung zu niedrig ist:
            if(compare & MASK_BATLOW)
            {
                RP6_readRegisters(RP6_BASE_ADR, I2C_REG_ADC_UBAT_L, messageBuf, 2);
                adcBat = (messageBuf[1] << 8) | (messageBuf[0]);

                handle_BatLow();
            }

            // ------------------------------------
            // Bewegung fertig? Notabschaltung?
            if(compare & MASK_DRIVE)
            {
                 movementComplete = messageBuf[2] & 1;
                 motorOvercurrent = messageBuf[2] & 4;
                 handle_DriveSystemChange();
            }

            // ------------------------------------
            // Auf Watchdog Requests prfen:
            if(messageBuf[1] & MASK_WDT_RQ)
            {
                handle_WDT_RQ();
            }
        }

        Thread_Delay(1);
    }
}

#endif
